SUBROUTINE diffusion (mat,sm,sub,S,dt)

  !----------------------------------------------------------------------------
  ! Resolution de la diffusion-reaction; la matrice de diffusion est completee
  ! et le systeme lineaire est ensuite resolu;
  ! LE TERME DE REACTION EST DEJA PRIS EN COMPTE DANS CALC_DIFF !!!
  !----------------------------------------------------------------------------

  USE mes_types
  USE param
  USE systeme_lineaire

  implicit none

  real,intent(in) :: dt

  TYPE(diag),DIMENSION(:),POINTER :: mat
  REAL,DIMENSION(:,:),POINTER :: sub
  REAL,DIMENSION(:),POINTER :: sm
  REAL,DIMENSION(:,:),POINTER :: S
  !TYPE (puits) :: S

  INTEGER :: i,j,k,l,m
  REAL :: d11,d22,d12m,d12p,val

  ! Fabrication de la matrice
  ! la diagonale principale est enregistree dans un tableau annexe
  !----------------------------------------------------------------
  store_mat=mat(1)%tab

  ! Complete matrix
  !----------------
  DO j=1,my
     DO i=1,mx
        m=mx*(j-1)+i
        ! Matrix
        mat(1)%tab(m)=mat(1)%tab(m)+deltax(i)*deltay(j)/dt
        ! Right-hand side
        scdm(m)=sm(m)+deltax(i)*deltay(j)*sub(i,j)/dt
     END DO
  END DO

  ! Coefficient that is not in any of the nine diagonals
  !------------------------------------------------------
  weight(1)=max(0.0,dt-C%tau(1))/dt
  weight(2)=max(0.0,dt-C%tau(2))/dt

  ! Re-injection of the mass after the refractory period
  !-----------------------------------------------------
  IF (C%tau(1)>zero) then
     IF (T+dt>C%tau(1)) then
        i=ib
        ref_times=ref%time(1,:)
        DO j=1,my
           m=mx*(j-1)+i
           ref_fluxes=ref%rflux(1,:,j)
           val=p_integral(ref_times,ref_fluxes,&
                &max(0.0,T-C%tau(1)),min(T,Tplus-C%tau(1)),ref%counter(1))
           ref%mass_out=ref%mass_out-val
           scdm(m)=scdm(m)+val/dt
        END DO
     END IF
  END IF

  IF (C%tau(2)>zero) then
     IF (T+dt>C%tau(2)) then
        j=jb
        ref_times=ref%time(2,:)
        DO i=1,mx
           m=mx*(j-1)+i
           ref_fluxes=ref%rflux(2,:,i)
           val=p_integral(ref_times,ref_fluxes,&
                &max(0.0,T-C%tau(2)),min(T,Tplus-C%tau(2)),ref%counter(2))
           ref%mass_out=ref%mass_out-val
           scdm(m)=scdm(m)+val/dt
        END DO
     END IF
  END IF

  ! Resolution du systeme lineaire
  !-------------------------------
  CALL syslin (mat,scdm,sol_1D,sol_1D,mtotal,test_cv_diff,&
       method_syslin_diff,diff_precond,(/1,2,4,6,8/))

  IF ((C%tau(1)>1d-15).and.(weight(1)>1d-15)) then
     DO j=1,my
        ref%mass_out=ref%mass_out-weight(1)*dt*C%diffusion(mx,j)&
             &*deltay(j)*C%val(mx,j)/(deltax(mx)/2.0)
     END DO
  END IF
  IF ((C%tau(2)>1d-15).and.(weight(2)>1d-15)) then
     DO i=1,mx
        ref%mass_out=ref%mass_out-weight(2)*dt*C%diffusion(i,my)&
             &*deltax(i)*C%val(i,my)/(deltay(my)/2.0)
     END DO
  END IF


  ! Post traitement
  !----------------
  DO l=1,mtotal
     sub(bij_tab(l,1),bij_tab(l,2))=sol_1D(l)
  END DO

  mat(1)%tab=store_mat

END SUBROUTINE diffusion
