MODULE functions

  !---------------------------------------------------------------------
  ! Role : contient toutes les fonctions du code
  !---------------------------------------------------------------------

  USE param

  implicit none

CONTAINS

  FUNCTION lireligne(uread)

    !---------------------------------------------------------------------
    ! Role : lit une ligne du fichier d'unite uread en ignorant les lignes 
    !  vides les lignes commencant par * ou #
    !---------------------------------------------------------------------

    IMPLICIT NONE

    CHARACTER(len=80)    :: lireligne
    INTEGER, INTENT(in)  :: uread

    CHARACTER(len=80)    :: buffer
    INTEGER :: i

    boucle :  DO
       READ(uread,'(A)', err = 1001, END = 1000) buffer
       buffer=ADJUSTL(buffer)

       SELECT CASE(buffer(:1))
       CASE ('#') 
          CONTINUE
       CASE ('*')
          CONTINUE
       CASE (' ')
          CONTINUE
       CASE default
          EXIT
       END SELECT
    END DO boucle
    i=1
   DO 
     IF (buffer(i:i)=='=') THEN
          buffer(i:i)=' '
          EXIT
       END IF
       buffer(i:i)=' '
       i=i+1
    END DO

10  lireligne = TRIM(buffer)
    !print*,'sortie'
    RETURN
1000 PRINT *, 'LIRELIGNE : fin de fichier '
    STOP
1001 PRINT *, 'LIRELIGNE : Erreur, contenu du tampon ; ', buffer
    STOP
  END FUNCTION lireligne

  FUNCTION lireligne2(uread)

    !---------------------------------------------------------------------
    ! Role : lit une ligne du fichier d'unite uread en ignorant les lignes 
    !  vides les lignes commencant par * ou #
    !---------------------------------------------------------------------

    IMPLICIT NONE

    CHARACTER(len=80)    :: lireligne2
    INTEGER, INTENT(in)  :: uread

    CHARACTER(len=80)    :: buffer
    INTEGER :: i

    boucle :  DO
       READ(uread,'(A)', err = 1001, END = 1000) buffer
       buffer=ADJUSTL(buffer)

       SELECT CASE(buffer(:1))
       CASE ('#') 
          CONTINUE
       CASE ('*')
          CONTINUE
       CASE (' ')
          CONTINUE
       CASE default
          EXIT
       END SELECT
    END DO boucle
    
10  lireligne2 = TRIM(buffer)
    !print*,'sortie'
    RETURN
1000 PRINT *, 'LIRELIGNE2 : fin de fichier '
    STOP
1001 PRINT *, 'LIRELIGNE2 : Erreur, contenu du tampon ; ', buffer
    STOP
  END FUNCTION lireligne2

  FUNCTION roe (x,par)

    REAL,INTENT(in) :: x,par
    REAL :: roe

    roe=MAX(0.0,MIN(par*x,1.0),MIN(x,par))

  END FUNCTION roe

  FUNCTION van_leer (x)

    REAL,INTENT(in) :: x
    REAL :: van_leer

    van_leer=(x+ABS(x))/(1+ABS(x))

  END FUNCTION van_leer
 

  FUNCTION bruneau (x)

    REAL,INTENT(in) :: x
    REAL :: bruneau

    bruneau=1.0
    IF (x/=1.0) bruneau=1-MIN(ABS(1.0-x),1/ABS(1.0-x))*(1.0-x)

!!$bruneau=max(0.0,x*(2.0-x),min(2.0,x,1.0+(1.0-x)**2))

  END FUNCTION bruneau


  FUNCTION mat_dot_vec (mat,vec,siz)

    USE mes_types

    TYPE (diag),dimension(:),pointer :: mat
    REAL,DIMENSION(1:),INTENT(IN) :: vec
    INTEGER,INTENT(IN) :: siz

    INTEGER :: i,j,s,m
    REAL,DIMENSION(siz) :: mat_dot_vec
    REAL :: d12p,d12m,d11,d22,val

    s=size(mat)/2+1

    !mat_dot_vec=0.0
    !mat_dot_vec=mat_dot_vec+mat(1)%tab*vec

    mat_dot_vec=mat(1)%tab*vec
   
    do i=2,s
       mat_dot_vec(1:siz-mat(i)%dist)=mat_dot_vec(1:siz-mat(i)%dist)&
            &+mat(i)%tab(1:siz-mat(i)%dist)*vec(1+mat(i)%dist:siz) 
       j=s+i-1
       mat_dot_vec(1+mat(j)%dist:siz)=mat_dot_vec(1+mat(j)%dist:siz)&
            &+mat(j)%tab(1+mat(j)%dist:siz)*vec(1:siz-mat(j)%dist) 
    end do


    ! Normal derivatives
    i=ib
    DO j=1,my
       m=(j-1)*mx+i
       d11=diff11(mx,j,C%diffusion)*deltay(j)/(deltax(mx)/2.0)
       mat_dot_vec(m)=mat_dot_vec(m)-d11*vec(j*mx)*weight(1)
    END DO

    j=jb
    DO i=1,mx
       m=(j-1)*mx+i
       d22=diff22(i,my,C%diffusion)*deltax(i)/(deltay(my)/2.0)
       mat_dot_vec(m)=mat_dot_vec(m)-d22*vec((my-1)*mx+i)*weight(2)
    END DO


!!$    ! Cross derivatives
!!$    ! interior points
!!$    i=ib
!!$    DO j=2,my-1
!!$       m=(j-1)*mx+i
!!$       d12p=diff12(i,j,C%diffusion)*deltay(j)/dyp(j)
!!$       d12m=diff12(i,j-1,C%diffusion)*deltay(j)/dyp(j-1)
!!$       val=.5&
!!$            &*(d12p*(jPstar(j+1,1)*vec(m-i+1+mx)-jPstar(j,1)*vec(m-i+1))&
!!$            &+d12m*(jPstar(j,1)*vec(m-i+1)-jPstar(j-1,1)*vec(m-i+1-mx)))
!!$       mat_dot_vec(m)=mat_dot_vec(m)-val
!!$       mat_dot_vec(m+1)=mat_dot_vec(m+1)+val
!!$       
!!$    END DO
!!$ 
!!$    ! Bottom
!!$    i=ib
!!$    j=1
!!$    m=(j-1)*mx+i
!!$    d12p=diff12(i,j,C%diffusion)*deltay(j)/dyp(j)
!!$    d12m=diff12(i,j-1,C%diffusion)*deltay(j)/(deltay(j)/2.0)
!!$    val=.5&
!!$         &*(d12p*(jPstar(j+1,1)*vec(m-i+1+mx)-jPstar(j,1)*vec(m-i+1))&
!!$         &+d12m*(jPstar(j,1)*vec(m-i+1)-0.0))
!!$    mat_dot_vec(m)=mat_dot_vec(m)-val
!!$    mat_dot_vec(m+1)=mat_dot_vec(m+1)+val
!!$    
!!$    
!!$    ! Top
!!$    i=ib
!!$    j=my
!!$    m=(j-1)*mx+i
!!$    d12p=diff12(i,j,C%diffusion)*deltay(j)/(deltay(j)/2.0)
!!$    d12m=diff12(i,j-1,C%diffusion)*deltay(j)/dyp(j-1)
!!$    val=.5&
!!$         &*(d12p*0.0&
!!$         &+d12m*(jPstar(j,1)*vec(m-i+1)-jPstar(j-1,1)*vec(m-i+1-mx)))
!!$    mat_dot_vec(m)=mat_dot_vec(m)-val
!!$    mat_dot_vec(m+1)=mat_dot_vec(m+1)+val
!!$    
!!$    
!!$    
!!$    j=jb
!!$    DO i=2,mx-1
!!$       m=(j-1)*mx+i
!!$       d12p=diff12(i,j,C%diffusion)*deltax(i)/dxp(i)
!!$       d12m=diff12(i-1,j,C%diffusion)*deltax(i)/dxp(i-1)
!!$       val=.5&
!!$            &*(d12p*(iPstar(i+1,1)*vec(i+1)-iPstar(i,1)*vec(i))&
!!$            &+d12m*(iPstar(i,1)*vec(i)-iPstar(i-1,1)*vec(i-1)))
!!$       mat_dot_vec(m)=mat_dot_vec(m)-val
!!$       m=m+mx
!!$       mat_dot_vec(m)=mat_dot_vec(m)+val
!!$    END DO
!!$    
!!$    !left
!!$    i=1
!!$    j=jb
!!$    m=(j-1)*mx+i
!!$    d12p=diff12(i,j,C%diffusion)*deltax(i)/dxp(i)
!!$    d12m=diff12(i-1,j,C%diffusion)*deltax(i)/(deltax(i)/2.0)
!!$    val=.5&
!!$         &*(d12p*(iPstar(i+1,1)*vec(i+1)-iPstar(i,1)*vec(i))&
!!$         &+d12m*(iPstar(i,1)*vec(i)-0.0))
!!$    mat_dot_vec(m)=mat_dot_vec(m)-val
!!$    m=m+mx
!!$    mat_dot_vec(m)=mat_dot_vec(m)+val
!!$    
!!$    ! right
!!$    i=mx
!!$    j=jb
!!$    m=(j-1)*mx+i
!!$    d12p=diff12(i,j,C%diffusion)*deltax(i)/(deltax(i)/2.0)
!!$    d12m=diff12(i-1,j,C%diffusion)*deltax(i)/dxp(i-1)
!!$    val=.5&
!!$         &*(d12p*0.0&
!!$         &+d12m*(iPstar(i,1)*vec(i)-iPstar(i-1,1)*vec(i-1)))
!!$    mat_dot_vec(m)=mat_dot_vec(m)-val
!!$    m=m+mx
!!$    mat_dot_vec(m)=mat_dot_vec(m)+val
    
    
!!$    DO i=1,siz      
!!$       DO j=mat%Jposi(i),mat%Jposi(i+1)-1
!!$          mat_dot_vec(i)=mat_dot_vec(i)+mat%TMat(j)*vec(mat%Jvcell(j))
!!$       END DO
!!$    END DO


  END FUNCTION mat_dot_vec

  FUNCTION mat_dot_vec_trans (mat,vec,siz)

    USE mes_types

    TYPE (diag),dimension(:),pointer :: mat
    REAL,DIMENSION(1:),INTENT(IN) :: vec
    INTEGER,INTENT(IN) :: siz

    INTEGER :: i,j,s,m
    REAL,DIMENSION(siz) :: mat_dot_vec_trans
    REAL :: d11,d12m,d22,d12p,val

    !print*,'entree'

    s=size(mat)/2+1

    mat_dot_vec_trans=mat(1)%tab*vec
   
    do i=2,s
       j=s+i-1
       mat_dot_vec_trans(1+mat(i)%dist:siz)&
            &=mat_dot_vec_trans(1+mat(i)%dist:siz)&
            &+mat(i)%tab(1:siz-mat(i)%dist)*vec(1:siz-mat(i)%dist)
       
       mat_dot_vec_trans(1:siz-mat(j)%dist)&
            &=mat_dot_vec_trans(1:siz-mat(j)%dist)&
            &+mat(j)%tab(1+mat(j)%dist:siz)*vec(1+mat(j)%dist:siz)      
    end do


    ! Normal derivatives
    i=ib
    DO j=1,my
       m=(j-1)*mx+i
       d11=diff11(mx,j,C%diffusion)*deltay(j)/(deltax(mx)/2.0)
       mat_dot_vec_trans(j*mx)=mat_dot_vec_trans(j*mx)-d11*vec(m)*weight(1)
    END DO

    j=jb
    DO i=1,mx
       m=(j-1)*mx+i
       d22=diff22(i,my,C%diffusion)*deltax(i)/(deltay(my)/2.0)
       mat_dot_vec_trans((my-1)*mx+i)=mat_dot_vec_trans((my-1)*mx+i)&
            &-d22*vec(m)*weight(2)
    END DO

!!$    ! Cross derivatives
!!$    ! interior points
!!$    i=ib
!!$    DO j=2,my-1
!!$       m=(j-1)*mx+i
!!$       d12p=diff12(i,j,C%diffusion)*deltay(j)/dyp(j)
!!$       d12m=diff12(i,j-1,C%diffusion)*deltay(j)/dyp(j-1)
!!$       
!!$       mat_dot_vec_trans(m-i+1+mx)=mat_dot_vec_trans(m-i+1+mx)&
!!$            &-.5*d12p*jPstar(j+1,1)*(vec(m)-vec(m+1))
!!$       mat_dot_vec_trans(m-i+1)=mat_dot_vec_trans(m-i+1)&
!!$            &-.5*(-d12p+d12m)*jPstar(j,1)*(vec(m)-vec(m+1))
!!$       mat_dot_vec_trans(m-i+1-mx)=mat_dot_vec_trans(m-i+1-mx)&
!!$            &-.5*d12m*(-jPstar(j-1,1))*(vec(m)-vec(m+1))
!!$    END DO
!!$ 
!!$    ! Bottom
!!$    i=ib
!!$    j=1
!!$    m=(j-1)*mx+i
!!$    d12p=diff12(i,j,C%diffusion)*deltay(j)/dyp(j)
!!$    d12m=diff12(i,j-1,C%diffusion)*deltay(j)/(deltay(j)/2.0)
!!$    mat_dot_vec_trans(m-i+1+mx)=mat_dot_vec_trans(m-i+1+mx)&
!!$         &-.5*d12p*jPstar(j+1,1)*(vec(m)-vec(m+1))
!!$    mat_dot_vec_trans(m-i+1)=mat_dot_vec_trans(m-i+1)&
!!$         &-.5*(-d12p+d12m)*jPstar(j,1)*(vec(m)-vec(m+1))
!!$    
!!$    
!!$    ! Top
!!$    i=ib
!!$    j=my
!!$    m=(j-1)*mx+i
!!$    d12p=diff12(i,j,C%diffusion)*deltay(j)/(deltay(j)/2.0)
!!$    d12m=diff12(i,j-1,C%diffusion)*deltay(j)/dyp(j-1)
!!$    mat_dot_vec_trans(m-i+1)=mat_dot_vec_trans(m-i+1)&
!!$         &-.5*(-0.0+d12m)*jPstar(j,1)*(vec(m)-vec(m+1))
!!$    mat_dot_vec_trans(m-i+1-mx)=mat_dot_vec_trans(m-i+1-mx)&
!!$         &-.5*d12m*(-jPstar(j-1,1))*(vec(m)-vec(m+1))
!!$    
!!$    
!!$    
!!$    j=jb
!!$    DO i=2,mx-1
!!$       m=(j-1)*mx+i
!!$       d12p=diff12(i,j,C%diffusion)*deltax(i)/dxp(i)
!!$       d12m=diff12(i-1,j,C%diffusion)*deltax(i)/dxp(i-1)
!!$       
!!$       mat_dot_vec_trans(i+1)=mat_dot_vec_trans(i+1)&
!!$            &-.5*iPstar(i+1,1)*d12p*(vec(m)-vec(m+mx))
!!$       mat_dot_vec_trans(i)=mat_dot_vec_trans(i)&
!!$            &-.5*iPstar(i,1)*(-d12p+d12m)*(vec(m)-vec(m+mx))
!!$       mat_dot_vec_trans(i-1)=mat_dot_vec_trans(i-1)&
!!$            &-.5*(-iPstar(i-1,1))*d12m*(vec(m)-vec(m+mx))
!!$    END DO
!!$    
!!$    !left
!!$    i=1
!!$    j=jb
!!$    m=(j-1)*mx+i
!!$    d12p=diff12(i,j,C%diffusion)*deltax(i)/dxp(i)
!!$    d12m=diff12(i-1,j,C%diffusion)*deltax(i)/(deltax(i)/2.0)
!!$    mat_dot_vec_trans(i+1)=mat_dot_vec_trans(i+1)&
!!$         &-.5*iPstar(i+1,1)*d12p*(vec(m)-vec(m+mx))
!!$    mat_dot_vec_trans(i)=mat_dot_vec_trans(i)&
!!$         &-.5*iPstar(i,1)*(-d12p+d12m)*(vec(m)-vec(m+mx))
!!$
!!$    
!!$    ! right
!!$    i=mx
!!$    j=jb
!!$    m=(j-1)*mx+i
!!$    d12p=diff12(i,j,C%diffusion)*deltax(i)/(deltax(i)/2.0)
!!$    d12m=diff12(i-1,j,C%diffusion)*deltax(i)/dxp(i-1)
!!$    mat_dot_vec_trans(i)=mat_dot_vec_trans(i)&
!!$         &-.5*iPstar(i,1)*(d12m)*(vec(m)-vec(m+mx))
!!$    mat_dot_vec_trans(i-1)=mat_dot_vec_trans(i-1)&
!!$         &-.5*(-iPstar(i-1,1))*d12m*(vec(m)-vec(m+mx))
    


!!$    mat_dot_vec_trans=0.0
!!$    DO i=1,siz
!!$       DO j=mat%Jposi(i),mat%Jposi(i+1)-1
!!$          mat_dot_vec_trans(mat%Jvcell(j))&
!!$               &=mat_dot_vec_trans(mat%Jvcell(j))+mat%TMat(j)*vec(i)
!!$       END DO
!!$    END DO

  END FUNCTION mat_dot_vec_trans

  FUNCTION mat_dot_vec_pointer (mat,vec,siz)

    USE mes_types

    TYPE (diag),dimension(:),pointer :: mat
    REAL,DIMENSION(:),POINTER :: vec
    INTEGER,INTENT(IN) :: siz

    INTEGER :: i,j,s,m
    REAL,DIMENSION(siz) :: mat_dot_vec_pointer
    REAL :: d12p,d12m,d11,d22,val

    s=size(mat)/2+1

!!$    mat_dot_vec_pointer=0.0
!!$    mat_dot_vec_pointer=mat_dot_vec_pointer+mat(1)%tab*vec

    mat_dot_vec_pointer=mat(1)%tab*vec
   
    do i=2,s
      mat_dot_vec_pointer(1:siz-mat(i)%dist)=&
           &mat_dot_vec_pointer(1:siz-mat(i)%dist)&
           &+mat(i)%tab(1:siz-mat(i)%dist)*vec(1+mat(i)%dist:siz) 
      j=s+i-1
      mat_dot_vec_pointer(1+mat(j)%dist:siz)=&
           &mat_dot_vec_pointer(1+mat(j)%dist:siz)&
           &+mat(j)%tab(1+mat(j)%dist:siz)*vec(1:siz-mat(j)%dist) 
   end do
   

   ! Normal derivatives
   i=ib
    DO j=1,my
       m=(j-1)*mx+i
       d11=diff11(mx,j,C%diffusion)*deltay(j)/(deltax(mx)/2.0)
       mat_dot_vec_pointer(m)=mat_dot_vec_pointer(m)-d11*vec(j*mx)*weight(1)
    END DO

    j=jb
    DO i=1,mx
       m=(j-1)*mx+i
       d22=diff22(i,my,C%diffusion)*deltax(i)/(deltay(my)/2.0)
       mat_dot_vec_pointer(m)=mat_dot_vec_pointer(m)&
            &-d22*vec((my-1)*mx+i)*weight(2)
    END DO

    



!!$    ! Cross derivatives
!!$    ! interior points
!!$    i=ib
!!$    DO j=2,my-1
!!$       m=(j-1)*mx+i
!!$       d12p=diff12(i,j,C%diffusion)*deltay(j)/dyp(j)
!!$       d12m=diff12(i,j-1,C%diffusion)*deltay(j)/dyp(j-1)
!!$       val=.5&
!!$            &*(d12p*(jPstar(j+1,1)*vec(m-i+1+mx)-jPstar(j,1)*vec(m-i+1))&
!!$            &+d12m*(jPstar(j,1)*vec(m-i+1)-jPstar(j-1,1)*vec(m-i+1-mx)))
!!$       mat_dot_vec_pointer(m)=mat_dot_vec_pointer(m)-val
!!$       mat_dot_vec_pointer(m+1)=mat_dot_vec_pointer(m+1)+val
!!$    END DO
!!$ 
!!$    ! Bottom
!!$    i=ib
!!$    j=1
!!$    m=(j-1)*mx+i
!!$    d12p=diff12(i,j,C%diffusion)*deltay(j)/dyp(j)
!!$    d12m=diff12(i,j-1,C%diffusion)*deltay(j)/(deltay(j)/2.0)
!!$    val=.5&
!!$         &*(d12p*(jPstar(j+1,1)*vec(m-i+1+mx)-jPstar(j,1)*vec(m-i+1))&
!!$         &+d12m*(jPstar(j,1)*vec(m-i+1)-0.0))
!!$    mat_dot_vec_pointer(m)=mat_dot_vec_pointer(m)-val
!!$    mat_dot_vec_pointer(m+1)=mat_dot_vec_pointer(m+1)+val
!!$    
!!$    
!!$    ! Top
!!$    i=ib
!!$    j=my
!!$    m=(j-1)*mx+i
!!$    d12p=diff12(i,j,C%diffusion)*deltay(j)/(deltay(j)/2.0)
!!$    d12m=diff12(i,j-1,C%diffusion)*deltay(j)/dyp(j-1)
!!$    val=.5&
!!$         &*(d12p*0.0&
!!$         &+d12m*(jPstar(j,1)*vec(m-i+1)-jPstar(j-1,1)*vec(m-i+1-mx)))
!!$    mat_dot_vec_pointer(m)=mat_dot_vec_pointer(m)-val
!!$    mat_dot_vec_pointer(m+1)=mat_dot_vec_pointer(m+1)+val
!!$    
!!$    
!!$    
!!$    j=jb
!!$    DO i=2,mx-1
!!$       m=(j-1)*mx+i
!!$       d12p=diff12(i,j,C%diffusion)*deltax(i)/dxp(i)
!!$       d12m=diff12(i-1,j,C%diffusion)*deltax(i)/dxp(i-1)
!!$       val=.5&
!!$            &*(d12p*(iPstar(i+1,1)*vec(i+1)-iPstar(i,1)*vec(i))&
!!$            &+d12m*(iPstar(i,1)*vec(i)-iPstar(i-1,1)*vec(i-1)))
!!$       mat_dot_vec_pointer(m)=mat_dot_vec_pointer(m)-val
!!$       m=m+mx
!!$       mat_dot_vec_pointer(m)=mat_dot_vec_pointer(m)+val
!!$    END DO
!!$    
!!$    !left
!!$    i=1
!!$    j=jb
!!$    m=(j-1)*mx+i
!!$    d12p=diff12(i,j,C%diffusion)*deltax(i)/dxp(i)
!!$    d12m=diff12(i-1,j,C%diffusion)*deltax(i)/(deltax(i)/2.0)
!!$    val=.5&
!!$         &*(d12p*(iPstar(i+1,1)*vec(i+1)-iPstar(i,1)*vec(i))&
!!$         &+d12m*(iPstar(i,1)*vec(i)-0.0))
!!$    mat_dot_vec_pointer(m)=mat_dot_vec_pointer(m)-val
!!$    m=m+mx
!!$    mat_dot_vec_pointer(m)=mat_dot_vec_pointer(m)+val
!!$    
!!$    ! right
!!$    i=mx
!!$    j=jb
!!$    m=(j-1)*mx+i
!!$    d12p=diff12(i,j,C%diffusion)*deltax(i)/(deltax(i)/2.0)
!!$    d12m=diff12(i-1,j,C%diffusion)*deltax(i)/dxp(i-1)
!!$    val=.5&
!!$         &*(d12p*0.0&
!!$         &+d12m*(iPstar(i,1)*vec(i)-iPstar(i-1,1)*vec(i-1)))
!!$    mat_dot_vec_pointer(m)=mat_dot_vec_pointer(m)-val
!!$    m=m+mx
!!$    mat_dot_vec_pointer(m)=mat_dot_vec_pointer(m)+val



 
!!$    mat_dot_vec_pointer=0.0
!!$    DO i=1,siz
!!$       DO j=mat%Jposi(i),mat%Jposi(i+1)-1
!!$          mat_dot_vec_pointer(i)=mat_dot_vec_pointer(i)+mat%TMat(j)&
!!$               &*vec(mat%Jvcell(j))
!!$       END DO
!!$    END DO

  END FUNCTION mat_dot_vec_pointer

  FUNCTION mat_dot_vec_trans_pointer (mat,vec,siz)

    USE mes_types

    TYPE (diag),dimension(:),pointer :: mat
    REAL,DIMENSION(:),pointer :: vec
    INTEGER,INTENT(IN) :: siz

    INTEGER :: i,j,s,m
    REAL,DIMENSION(siz) :: mat_dot_vec_trans_pointer
    REAL :: d12p,d12m,d11,d22,val

!!$    mat_dot_vec_trans_pointer=0.0
!!$    mat_dot_vec_trans_pointer=mat_dot_vec_trans_pointer+mat(1)%tab*vec


    s=size(mat)/2+1

    mat_dot_vec_trans_pointer=mat(1)%tab*vec
   
    do i=2,s
       j=s+i-1
       mat_dot_vec_trans_pointer(1+mat(i)%dist:siz)&
            &=mat_dot_vec_trans_pointer(1+mat(i)%dist:siz)&
            &+mat(i)%tab(1:siz-mat(i)%dist)*vec(1:siz-mat(i)%dist)
       
       mat_dot_vec_trans_pointer(1:siz-mat(j)%dist)&
            &=mat_dot_vec_trans_pointer(1:siz-mat(j)%dist)&
            &+mat(j)%tab(1+mat(j)%dist:siz)*vec(1+mat(j)%dist:siz)      
    end do


    ! Normal derivatives
    i=ib
    DO j=1,my
       m=(j-1)*mx+i
       d11=diff11(mx,j,C%diffusion)*deltay(j)/(deltax(mx)/2.0)
       mat_dot_vec_trans_pointer(j*mx)=mat_dot_vec_trans_pointer(j*mx)&
            &-d11*vec(m)*weight(1)
    END DO

    j=jb
    DO i=1,mx
       m=(j-1)*mx+i
       d22=diff22(i,my,C%diffusion)*deltax(i)/(deltay(my)/2.0)
       mat_dot_vec_trans_pointer((my-1)*mx+i)&
            &=mat_dot_vec_trans_pointer((my-1)*mx+i)-d22*vec(m)*weight(2)
    END DO


!!$    ! Cross derivatives
!!$    ! interior points
!!$    i=ib
!!$    DO j=2,my-1
!!$       m=(j-1)*mx+i
!!$       d12p=diff12(i,j,C%diffusion)*deltay(j)/dyp(j)
!!$       d12m=diff12(i,j-1,C%diffusion)*deltay(j)/dyp(j-1)
!!$       
!!$       mat_dot_vec_trans_pointer(m-i+1+mx)=mat_dot_vec_trans_pointer(m-i+1+mx)&
!!$            &-.5*d12p*jPstar(j+1,1)*(vec(m)-vec(m+1))
!!$       mat_dot_vec_trans_pointer(m-i+1)=mat_dot_vec_trans_pointer(m-i+1)&
!!$            &-.5*(-d12p+d12m)*jPstar(j,1)*(vec(m)-vec(m+1))
!!$       mat_dot_vec_trans_pointer(m-i+1-mx)=mat_dot_vec_trans_pointer(m-i+1-mx)&
!!$            &-.5*d12m*(-jPstar(j-1,1))*(vec(m)-vec(m+1))
!!$    END DO
!!$ 
!!$    ! Bottom
!!$    i=ib
!!$    j=1
!!$    m=(j-1)*mx+i
!!$    d12p=diff12(i,j,C%diffusion)*deltay(j)/dyp(j)
!!$    d12m=diff12(i,j-1,C%diffusion)*deltay(j)/(deltay(j)/2.0)
!!$    mat_dot_vec_trans_pointer(m-i+1+mx)=mat_dot_vec_trans_pointer(m-i+1+mx)&
!!$         &-.5*d12p*jPstar(j+1,1)*(vec(m)-vec(m+1))
!!$    mat_dot_vec_trans_pointer(m-i+1)=mat_dot_vec_trans_pointer(m-i+1)&
!!$         &-.5*(-d12p+d12m)*jPstar(j,1)*(vec(m)-vec(m+1))
!!$    
!!$    
!!$    ! Top
!!$    i=ib
!!$    j=my
!!$    m=(j-1)*mx+i
!!$    d12p=diff12(i,j,C%diffusion)*deltay(j)/(deltay(j)/2.0)
!!$    d12m=diff12(i,j-1,C%diffusion)*deltay(j)/dyp(j-1)
!!$    mat_dot_vec_trans_pointer(m-i+1)=mat_dot_vec_trans_pointer(m-i+1)&
!!$         &-.5*(-0.0+d12m)*jPstar(j,1)*(vec(m)-vec(m+1))
!!$    mat_dot_vec_trans_pointer(m-i+1-mx)=mat_dot_vec_trans_pointer(m-i+1-mx)&
!!$         &-.5*d12m*(-jPstar(j-1,1))*(vec(m)-vec(m+1))
!!$    
!!$    
!!$    
!!$    j=jb
!!$    DO i=2,mx-1
!!$       m=(j-1)*mx+i
!!$       d12p=diff12(i,j,C%diffusion)*deltax(i)/dxp(i)
!!$       d12m=diff12(i-1,j,C%diffusion)*deltax(i)/dxp(i-1)
!!$       
!!$       mat_dot_vec_trans_pointer(i+1)=mat_dot_vec_trans_pointer(i+1)&
!!$            &-.5*iPstar(i+1,1)*d12p*(vec(m)-vec(m+mx))
!!$       mat_dot_vec_trans_pointer(i)=mat_dot_vec_trans_pointer(i)&
!!$            &-.5*iPstar(i)*(-d12p+d12m)*(vec(m)-vec(m+mx))
!!$       mat_dot_vec_trans_pointer(i-1)=mat_dot_vec_trans_pointer(i-1)&
!!$            &-.5*(-iPstar(i-1,1))*d12m*(vec(m)-vec(m+mx))
!!$    END DO
!!$    
!!$    !left
!!$    i=1
!!$    j=jb
!!$    m=(j-1)*mx+i
!!$    d12p=diff12(i,j,C%diffusion)*deltax(i)/dxp(i)
!!$    d12m=diff12(i-1,j,C%diffusion)*deltax(i)/(deltax(i)/2.0)
!!$    mat_dot_vec_trans_pointer(i+1)=mat_dot_vec_trans_pointer(i+1)&
!!$         &-.5*iPstar(i+1,1)*d12p*(vec(m)-vec(m+mx))
!!$    mat_dot_vec_trans_pointer(i)=mat_dot_vec_trans_pointer(i)&
!!$         &-.5*iPstar(i)*(-d12p+d12m)*(vec(m)-vec(m+mx))
!!$
!!$    
!!$    ! right
!!$    i=mx
!!$    j=jb
!!$    m=(j-1)*mx+i
!!$    d12p=diff12(i,j,C%diffusion)*deltax(i)/(deltax(i)/2.0)
!!$    d12m=diff12(i-1,j,C%diffusion)*deltax(i)/dxp(i-1)
!!$    mat_dot_vec_trans_pointer(i)=mat_dot_vec_trans_pointer(i)&
!!$         &-.5*iPstar(i)*(d12m)*(vec(m)-vec(m+mx))
!!$    mat_dot_vec_trans_pointer(i-1)=mat_dot_vec_trans_pointer(i-1)&
!!$         &-.5*(-iPstar(i-1,1))*d12m*(vec(m)-vec(m+mx))



!!$    mat_dot_vec_trans_pointer=0.0
!!$    DO i=1,siz
!!$       DO j=mat%Jposi(i),mat%Jposi(i+1)-1
!!$          mat_dot_vec_trans_pointer(mat%Jvcell(j))&
!!$               &=mat_dot_vec_trans_pointer(mat%Jvcell(j))+mat%TMat(j)*vec(i)
!!$       END DO
!!$    END DO

  END FUNCTION mat_dot_vec_trans_pointer

  FUNCTION scalar(vec1,vec2)

    REAL,DIMENSION(1:) :: vec1,vec2
    REAL :: scalar

    scalar=SUM(vec1*vec2)

  END FUNCTION scalar


  FUNCTION scalar_pointer(vec1,vec2)

    REAL,DIMENSION(:),POINTER :: vec1,vec2
    REAL :: scalar_pointer

    scalar_pointer=SUM(vec1*vec2)

  END FUNCTION scalar_pointer


  function diff11 (i,j,tab)

    integer :: i,j
    real :: ci,cio
    real :: diff11
    real,dimension(:,:,:),pointer :: tab

    ci=1.0-xinterp(i)
    cio=xinterp(i)

    diff11=ci*tab(i,j,1)+cio*tab(i+1,j,1)

  end function diff11


  function diff22 (i,j,tab)

    integer :: i,j
    real :: cj,cjo
    real :: diff22
    real,dimension(:,:,:),pointer :: tab

    cjo=yinterp(j)
    cj=1.0-yinterp(j)

    diff22=cj*tab(i,j,2)+cjo*tab(i,j+1,2)
 
  end function diff22

  function diff12 (i,j,tab)

    integer :: i,j,k
    real :: ci,cio,cj,cjo
    real :: diff12
    real,dimension(:,:,:),pointer :: tab

    cio=xinterp(i)
    ci=1.0-xinterp(i)
    cjo=yinterp(j)
    cj=1.0-yinterp(j)
 
    diff12=ci*cj*tab(i,j,3)+cio*cj*tab(i+1,j,3)&
         &+ci*cjo*tab(i,j+1,3)+cio*cjo*tab(i+1,j+1,3)

    !print*,'diff12=',diff12
  end function diff12


  function p_integral (x,f,xmin,xmax,ind_max)

    real,intent(in) :: xmin,xmax
    integer,intent(in) :: ind_max
    real,dimension(:),pointer :: x,f
    real :: p_integral
    
    integer :: imin,imax,i,j1,j2

    p_integral=0.0


    IF (xmin+1e-15<xmax) THEN
       imin=1
       imax=ind_max
       i=1
       loop1 : DO i=1,ind_max
          if ((xmin>=x(i)).and.(xmin<x(i+1))) then
             imin=i
             exit loop1
          END if
       END DO loop1
       j1=i

       i=1
       loop2: DO i=1,ind_max
          if ((xmax>x(i)).and.(xmax<=x(i+1))) then
             imax=i
             exit loop2
          END if
       END DO loop2
       j2=i

       DO i=imin+1,imax-1
          p_integral=p_integral+(x(i+1)-x(i))*f(i)
       END DO
       IF (imin/=imax) THEN
          p_integral=p_integral+(x(imin+1)-xmin)*f(imin)+(xmax-x(imax))*f(imax)
       ELSE
          p_integral=p_integral+(xmax-xmin)*f(imin)
       END IF
    END IF

    if ((imin>400).or.(imax>400)) then
       print*,'val=',imin,imax,x(imin),x(imax),xmin,xmax
       stop
    end if
    
  end function p_integral


END MODULE functions







