MODULE alloc

  !--------------------------------------------------------------------------
  ! Goal : Contains all the allocation routines
  !
  ! SUBROUTINE alloue_integer_1D: allocate a 1D pointer of integers
  ! SUBROUTINE alloue_integer_1D: allocate a 2D pointer of integers
  ! SUBROUTINE alloue_integer_3D: allocate a 3D pointer of integers
  ! SUBROUTINE alloue_real_1D: allocate a 1D pointer of reals
  ! SUBROUTINE alloue_real_2D: allocate a 2D pointer of reals
  ! SUBROUTINE alloue_real_3D: allocate a 3D pointer of reals
  ! SUBROUTINE alloue_real_4D: allocate a 4D pointer of reals
  ! SUBROUTINE alloue_champ: allocate a 1D pointer of type champ
  ! SUBROUTINE alloue_logical_4D: allocate a 4D pointer of type logical
  ! SUBROUTINE alloue_character_1D: allocate a 1D pointer of type character
  ! SUBROUTINE alloue_cl_1D: allocate a 1D pointer of type bc
  ! SUBROUTINE alloue_mat_bande: allocate a 1D pointer of type diag
  ! SUBROUTINE erreur_alloc: Test whether there is an internal error associated
  !                          with allocation/deallocation.
  !--------------------------------------------------------------------------

  INTEGER :: test

CONTAINS

  SUBROUTINE alloue_integer_1D(tab,n1,n2,par)

    INTEGER,DIMENSION(:),POINTER :: tab
    INTEGER,INTENT(in) :: par,n1,n2
    INTEGER :: all
    IF (par==1) THEN 
       ALLOCATE(tab(n1:n2),stat=test)
    ELSE IF (par==0) THEN
       DEALLOCATE(tab,stat=test) 
    ELSE 
       PRINT*,"Allocation error"
       STOP
    END IF

    IF(test/=0) CALL erreur_alloc (par)

  END SUBROUTINE alloue_integer_1D


  SUBROUTINE alloue_integer_2D(tab,n1,n2,n3,n4,par)

    INTEGER,DIMENSION(:,:),POINTER :: tab
    INTEGER,INTENT(in) :: par,n1,n2,n3,n4
    IF (par==1) THEN 
       ALLOCATE(tab(n1:n2,n3:n4),stat=test)
    ELSE IF (par==0) THEN
       DEALLOCATE(tab,stat=test) 
    ELSE 
       PRINT*,"Allocation error"
       STOP

    END IF

    IF(test/=0) CALL erreur_alloc (par)
  END SUBROUTINE alloue_integer_2D


  SUBROUTINE alloue_integer_3D(tab,n1,n2,n3,n4,n5,n6,par)

    INTEGER,DIMENSION(:,:,:),POINTER :: tab
    INTEGER,INTENT(in) :: par,n1,n2,n3,n4,n5,n6
    IF (par==1) THEN 
       ALLOCATE(tab(n1:n2,n3:n4,n5:n6),stat=test)
    ELSE IF (par==0) THEN
       DEALLOCATE(tab,stat=test) 
    ELSE 
       PRINT*,"Allocation error"
       STOP

    END IF

    IF(test/=0) CALL erreur_alloc (par)
  END SUBROUTINE alloue_integer_3D



  SUBROUTINE alloue_real_1D(tab,nombre1,nombre2,par)

    REAL,DIMENSION(:),POINTER :: tab
    INTEGER,INTENT(in) :: par,nombre1,nombre2
    INTEGER :: all
    IF (par==1) THEN 
       ALLOCATE(tab(nombre1:nombre2),stat=test)
    ELSE IF (par==0) THEN
       DEALLOCATE(tab,stat=test) 
    ELSE 
       PRINT*,"Allocation error"
       STOP
    END IF

    IF(test/=0) CALL erreur_alloc (par)

  END SUBROUTINE alloue_real_1D

  SUBROUTINE alloue_int_1D(tab,nombre1,nombre2,par)

    INTEGER,DIMENSION(:),POINTER :: tab
    INTEGER,INTENT(in) :: par,nombre1,nombre2
    INTEGER :: all
    IF (par==1) THEN 
       ALLOCATE(tab(nombre1:nombre2),stat=test)
    ELSE IF (par==0) THEN
       DEALLOCATE(tab,stat=test) 
    ELSE 
       PRINT*,"Allocation error"
       STOP
    END IF

    IF(test/=0) CALL erreur_alloc (par)

  END SUBROUTINE alloue_int_1D


  SUBROUTINE alloue_real_2D(tab,nombre1,nombre2,nombre3,nombre4,par)

    REAL,DIMENSION(:,:),POINTER :: tab
    INTEGER,INTENT(in) :: par,nombre1,nombre2,nombre3,nombre4
    IF (par==1) THEN 
       ALLOCATE(tab(nombre1:nombre2,nombre3:nombre4),stat=test)
    ELSE IF (par==0) THEN
       DEALLOCATE(tab,stat=test) 
    ELSE 
       PRINT*,"Allocation error"
       STOP

    END IF

    IF(test/=0) CALL erreur_alloc (par)
  END SUBROUTINE alloue_real_2D

  SUBROUTINE alloue_real_3D(tab,n1,n2,n3,n4,n5,n6,par)

    REAL,DIMENSION(:,:,:),POINTER :: tab
    INTEGER,INTENT(in) :: par,n1,n2,n3,n4,n5,n6
    IF (par==1) THEN 
       ALLOCATE(tab(n1:n2,n3:n4,n5:n6),stat=test)
    ELSE IF (par==0) THEN
       DEALLOCATE(tab,stat=test) 
    ELSE 
       PRINT*,"Allocation error"
       STOP
    END IF

    IF(test/=0) CALL erreur_alloc (par)
  END SUBROUTINE alloue_real_3D

  SUBROUTINE alloue_real_4D(tab,n1,n2,n3,n4,n5,n6,n7,n8,par)

    REAL,DIMENSION(:,:,:,:),POINTER :: tab
    INTEGER,INTENT(in) :: par,n1,n2,n3,n4,n5,n6,n7,n8
    IF (par==1) THEN 
       ALLOCATE(tab(n1:n2,n3:n4,n5:n6,n7:n8),stat=test)
    ELSE IF (par==0) THEN
       DEALLOCATE(tab,stat=test) 
    ELSE 
       PRINT*,"Allocation error"
       STOP

    END IF

    IF(test/=0) CALL erreur_alloc (par)
  END SUBROUTINE alloue_real_4D

  SUBROUTINE alloue_champ(tab,nombre1,nombre2,par)

    USE mes_types

    TYPE(champ),DIMENSION(:),POINTER :: tab
    INTEGER,INTENT(in) :: par,nombre1,nombre2
    IF (par==1) THEN 
       ALLOCATE(tab(nombre1:nombre2),stat=test)
    ELSE IF (par==0) THEN
       DEALLOCATE(tab,stat=test) 
    ELSE 
       PRINT*,"Allocation error"
       STOP

    END IF

    IF(test/=0) CALL erreur_alloc (par)
  END SUBROUTINE alloue_champ

  SUBROUTINE alloue_logical_4D(tab,n1,n2,n3,n4,n5,n6,n7,n8,par)

    LOGICAL,DIMENSION(:,:,:,:),POINTER :: tab
    INTEGER,INTENT(in) :: par,n1,n2,n3,n4,n5,n6,n7,n8
    IF (par==1) THEN 
       ALLOCATE(tab(n1:n2,n3:n4,n5:n6,n7:n8),stat=test)
    ELSE IF (par==0) THEN
       DEALLOCATE(tab,stat=test) 
    ELSE 
       PRINT*,"Allocation error"
       STOP

    END IF

    IF(test/=0) CALL erreur_alloc (par)
  END SUBROUTINE alloue_logical_4D

  SUBROUTINE alloue_character_1D(tab,n1,n2,par)

    CHARACTER,DIMENSION(:),POINTER :: tab
    INTEGER,INTENT(in) :: par,n1,n2
    IF (par==1) THEN 
       ALLOCATE(tab(n1:n2),stat=test)
    ELSE IF (par==0) THEN
       DEALLOCATE(tab,stat=test) 
    ELSE 
       PRINT*,"Allocation error"
       STOP
    END IF

    IF(test/=0) CALL erreur_alloc (par)
  END SUBROUTINE alloue_character_1D

  SUBROUTINE alloue_cl_1D(tab,n1,n2,par)

    USE mes_types

    TYPE(cl),DIMENSION(:),POINTER :: tab
    INTEGER,INTENT(in) :: par,n1,n2
    IF (par==1) THEN 
       ALLOCATE(tab(n1:n2),stat=test)
    ELSE IF (par==0) THEN
       DEALLOCATE(tab,stat=test) 
    ELSE 
       PRINT*,"Allocation error"
       STOP
    END IF

    IF(test/=0) CALL erreur_alloc (par)
  END SUBROUTINE alloue_cl_1D

  SUBROUTINE alloue_character_len_2D(tab,n1,n2,n3,n4,par)

    CHARACTER(len=*),DIMENSION(:,:),POINTER :: tab
    INTEGER,INTENT(in) :: par,n1,n2,n3,n4
    IF (par==1) THEN 
       ALLOCATE(tab(n1:n2,n3:n4),stat=test)
    ELSE IF (par==0) THEN
       DEALLOCATE(tab,stat=test) 
    ELSE 
       PRINT*,"Allocation error"
       STOP
    END IF

    IF(test/=0) CALL erreur_alloc (par)
  END SUBROUTINE alloue_character_len_2D


  SUBROUTINE alloue_mat_bande(tab,n1,n2,par)

    USE mes_types
    TYPE(diag),DIMENSION(:),POINTER :: tab
    INTEGER,INTENT(in) :: par,n1,n2
    IF (par==1) THEN 
       ALLOCATE(tab(n1:n2),stat=test)
    ELSE IF (par==0) THEN
       DEALLOCATE(tab,stat=test) 
    ELSE 
       PRINT*,"Allocation error"
       STOP
    END IF

    IF(test/=0) CALL erreur_alloc (par)

  END SUBROUTINE alloue_mat_bande


  SUBROUTINE erreur_alloc (n)

    INTEGER,INTENT(in) :: n
    IF (n==1) PRINT*,"Allocation error: Memory full"
    IF (n==0) PRINT*,"Deallocation error: Table not allocated"
    STOP
  END SUBROUTINE erreur_alloc

END MODULE alloc
