SUBROUTINE diffusion (mat,sm,sub,S,dt)

  !---------------------------------------------------------------------
  ! Goal: Iterate the pure diffusion scheme
  !---------------------------------------------------------------------

  USE mes_types
  USE param
  USE systeme_lineaire

  IMPLICIT NONE

  REAL,INTENT(in) :: dt

  TYPE(diag),DIMENSION(:),POINTER :: mat
  REAL,DIMENSION(:),POINTER :: sub
  REAL,DIMENSION(:),POINTER :: sm,S

  INTEGER :: i,j,k,l,m
  REAL :: ci,cio,d11,val,val2,T_bar

  ! Complete matrix and right-hand side
  !------------------------------------
  DO i=1,mx
     m=i
     ! Matrix
     !--------
     mat(1)%tab(m)=mat(1)%tab(m)+deltax(i)/dt
     ! Right-hand side
     !----------------
     scdm(m)=sm(m)+deltax(i)*sub(i)/dt
  END DO

  ! Coefficient that is not in any of the three diagonals
  !------------------------------------------------------
  weight=MAX(0.0,dt-C%tau)/dt
  
  ! Re-injection of the mass after the refractory period
  !-----------------------------------------------------
  IF (C%tau>1d-15) THEN
     IF (T+dt>C%tau) THEN
        i=ib
        val=p_integral(ref%time,ref%rflux,&
             &MAX(0.0,C%tau-dt),C%tau)/dt
        mass_out=mass_out-val*dt
        scdm(ib)=scdm(ib)+val
     END IF
  END IF

!!$        ! This works for absorbing condition
!!$        i=1
!!$        m=i
!!$        d11=diff11(i-1,C%diffusion)/(dxp(i-1)/2.0)
!!$        val=2.0*Vx(1)*dt/deltax(1)
!!$        mat(1)%tab(m)=mat(1)%tab(m)+d11*(1.0+val/(1.0-val))
!!$        scdm(m)=scdm(m)+d11*bcl_save/(1.0-val)
!!$ 
!!$        i=mx
!!$        m=i
!!$        d11=diff11(i,C%diffusion)/(dxp(i)/2.0)
!!$        val=2.0*Vx(mx)*dt/deltax(mx)
!!$        val2=1.0-val+Vx(mx)**2*dt/diff11(i,C%diffusion)
!!$        mat(1)%tab(m)=mat(1)%tab(m)+d11*(1.0+val/val2)
!!$        scdm(m)=scdm(m)+d11*bcr_save/val2

  ! Solve the linear system
  !-------------------------
  CALL syslin (mat,scdm,sol_1D,sol_1D,mx,test_cv_diff,&
       method_syslin_diff,diff_precond,(/1,2,3,8,11,13,17/))

  ! Calculate the probability mass that crosses the threshold
  !-----------------------------------------------------------
  IF ((C%tau>1d-15).AND.(weight>1d-15)) THEN
     mass_out=mass_out-weight*dt*C%diffusion(mx)&
          &*sol_1D(mx)/(deltax(mx)/2.0)
  END IF

  ! Store into sub Post treatment
  !----------------
  DO i=1,mx
     sub(i)=sol_1D(i)
  END DO

END SUBROUTINE diffusion
