SUBROUTINE domain

  !------------------------------------------------------------------------
  ! Goal: Read data associated with the domain and construct mesh
  !------------------------------------------------------------------------

  USE functions
  USE files
  USE mes_types
  USE param
  USE alloc
  
  TYPE m1
     REAL :: xmin,xmax
     INTEGER :: nbr_mesh
  END TYPE m1

  INTEGER :: nbr_int,nbr_save
  REAL :: xmin,xmax,nbr_mesh,val
  TYPE (m1),DIMENSION(:),POINTER :: maille1

  CHARACTER(len=len_buffer) :: buffer
  INTEGER :: i,j,k,l
  REAL :: diffusion_loc

  !OPEN (unit=udom,file=FDOM,status='old')

  ! Read the mesh
  !---------------
  buffer=lireligne(uparam)
  READ(buffer, *, err = 10) nbr_int

  ALLOCATE (maille1(1:nbr_int))
  mx=0
  DO i=1,nbr_int
     buffer=lireligne(uparam)
     READ(buffer, *, err = 10) maille1(i)%xmin,maille1(i)%xmax,&
          &maille1(i)%nbr_mesh
     mx=mx+maille1(i)%nbr_mesh
  END DO
  nx=mx+1

  ! Allocate voltage steps and interpolation coefficients
  !-------------------------------------------------------
  CALL alloue_real_1D (deltax,1,mx,1)
  CALL alloue_real_1D (dxp,0,mx,1)
  CALL alloue_real_1D (xinterp,0,mx,1)

  ! Calculate voltage steps and interpolation coefficients
  !-------------------------------------------------------
  nbr_save=0
  DO i=1,nbr_int
     val=(maille1(i)%xmax-maille1(i)%xmin)/maille1(i)%nbr_mesh 
     DO j=1,maille1(i)%nbr_mesh
        deltax(nbr_save+j)=val
     END DO
     nbr_save=nbr_save+maille1(i)%nbr_mesh
  END DO
  dxp(0)=deltax(1)
  DO i=1,mx-1
     dxp(i)=.5*(deltax(i)+deltax(i+1))
  END DO
  dxp(mx)=deltax(mx)
  xinterp(0)=.5
  DO i=1,mx-1
     xinterp(i)=.5*deltax(i)/dxp(i)
  END DO
  xinterp(mx)=.5
  DEALLOCATE (maille1)

  ! Allocate mesh-points (xpoints) and positions of the interfaces
  !---------------------------------------------------------------
  ALLOCATE (xpoints(mx))
  ALLOCATE (xmesh(0:mx))
  xmesh(0)=xd_min
  xmesh(1)=xd_min+deltax(1)
  xpoints(1)=xd_min+.5*deltax(1)
  DO i=2,mx
     xmesh(i)=xmesh(i-1)+deltax(i)
     xpoints(i)=xpoints(i-1)+.5*(deltax(i-1)+deltax(i))
  END DO
  mtotal=mx

  ! Determine the index of the mesh containing the reset voltage
  !-------------------------------------------------------------
  DO i=1,mx
     IF ((xmesh(i-1)<=break).AND.(xmesh(i)>break)) ib=i
  END DO

  RETURN

10 PRINT*,'Error when reading data associated with the domain'
  CALL desallocation
  STOP

END SUBROUTINE domain









