MODULE functions

  !---------------------------------------------------------------------
  ! Role : contains all the function of the code
  !---------------------------------------------------------------------

  USE param

  implicit none

CONTAINS

  FUNCTION lireligne(uread)

    !---------------------------------------------------------------------
    ! Goal: read a line in the file of unit uread ignoring empty lines and 
    !       lines starting with either  * or #
    !---------------------------------------------------------------------

    IMPLICIT NONE

    CHARACTER(len=80)    :: lireligne
    INTEGER, INTENT(in)  :: uread

    CHARACTER(len=80)    :: buffer
    INTEGER :: i

    boucle :  DO
       READ(uread,'(A)', err = 1001, END = 1000) buffer
       buffer=ADJUSTL(buffer)

       SELECT CASE(buffer(:1))
       CASE ('#') 
          CONTINUE
       CASE ('*')
          CONTINUE
       CASE (' ')
          CONTINUE
       CASE default
          EXIT
       END SELECT
    END DO boucle
    i=1
    DO 
       IF (buffer(i:i)=='=') THEN
          buffer(i:i)=' '
          EXIT
       END IF
       buffer(i:i)=' '
       i=i+1
    END DO

10  lireligne = TRIM(buffer)
    !print*,'sortie'
    RETURN
1000 PRINT *, 'LIRELIGNE : fin de fichier '
    STOP
1001 PRINT *, 'LIRELIGNE : Erreur, contenu du tampon ; ', buffer
    STOP
  END FUNCTION lireligne

  FUNCTION mat_dot_vec (mat,vec,siz)

    !-----------------------------------------------------------------------
    ! Goal: Calculate the product of matrix mat and vector vec (of size siz)
    !       if vec is a table
    !-----------------------------------------------------------------------

    USE mes_types

    TYPE (diag),dimension(:),pointer :: mat
    REAL,DIMENSION(1:),INTENT(IN) :: vec
    INTEGER,INTENT(IN) :: siz

    INTEGER :: i,j,s
    REAL,DIMENSION(siz) :: mat_dot_vec
    REAL :: d11

    s=size(mat)/2+1

    mat_dot_vec=mat(1)%tab*vec

    do i=2,s
       mat_dot_vec(1:siz-mat(i)%dist)=mat_dot_vec(1:siz-mat(i)%dist)&
            &+mat(i)%tab(1:siz-mat(i)%dist)*vec(1+mat(i)%dist:siz) 
       j=s+i-1
       mat_dot_vec(1+mat(j)%dist:siz)=mat_dot_vec(1+mat(j)%dist:siz)&
            &+mat(j)%tab(1+mat(j)%dist:siz)*vec(1:siz-mat(j)%dist) 
    end do

    d11=diff11(mx,C%diffusion)/(deltax(mx)/2.0)
    mat_dot_vec(ib)=mat_dot_vec(ib)-d11*vec(mx)*weight

  END FUNCTION mat_dot_vec

  FUNCTION mat_dot_vec_trans (mat,vec,siz)

    !-----------------------------------------------------------------------
    ! Goal: Calculate the product of transpose of matrix mat 
    !       and vector vec (of size siz) if vec is a table
    !-----------------------------------------------------------------------

    USE mes_types

    TYPE (diag),dimension(:),pointer :: mat
    REAL,DIMENSION(1:),INTENT(IN) :: vec
    INTEGER,INTENT(IN) :: siz

    INTEGER :: i,j,s
    REAL,DIMENSION(siz) :: mat_dot_vec_trans
    REAL :: d11

    s=size(mat)/2+1

    mat_dot_vec_trans=mat(1)%tab*vec

    do i=2,s
       j=s+i-1
       mat_dot_vec_trans(1+mat(i)%dist:siz)&
            &=mat_dot_vec_trans(1+mat(i)%dist:siz)&
            &+mat(i)%tab(1:siz-mat(i)%dist)*vec(1:siz-mat(i)%dist)

       mat_dot_vec_trans(1:siz-mat(j)%dist)&
            &=mat_dot_vec_trans(1:siz-mat(j)%dist)&
            &+mat(j)%tab(1+mat(j)%dist:siz)*vec(1+mat(j)%dist:siz)      
    end do

    i=ib
    d11=diff11(mx,C%diffusion)/(deltax(mx)/2.0)
    mat_dot_vec_trans(mx)=mat_dot_vec_trans(mx)-d11*vec(i)*weight

  END FUNCTION mat_dot_vec_trans

  FUNCTION mat_dot_vec_pointer (mat,vec,siz)

    !-----------------------------------------------------------------------
    ! Goal: Calculate the product of matrix mat and vector vec (of size siz)
    !       if vec is a pointer
    !-----------------------------------------------------------------------

    USE mes_types

    TYPE (diag),dimension(:),pointer :: mat
    REAL,DIMENSION(:),POINTER :: vec
    INTEGER,INTENT(IN) :: siz

    INTEGER :: i,j,s
    REAL,DIMENSION(siz) :: mat_dot_vec_pointer
    REAL :: d11

    s=size(mat)/2+1

    mat_dot_vec_pointer=mat(1)%tab*vec

    do i=2,s
       mat_dot_vec_pointer(1:siz-mat(i)%dist)=&
            &mat_dot_vec_pointer(1:siz-mat(i)%dist)&
            &+mat(i)%tab(1:siz-mat(i)%dist)*vec(1+mat(i)%dist:siz) 
       j=s+i-1
       mat_dot_vec_pointer(1+mat(j)%dist:siz)=&
            &mat_dot_vec_pointer(1+mat(j)%dist:siz)&
            &+mat(j)%tab(1+mat(j)%dist:siz)*vec(1:siz-mat(j)%dist) 
    end do

    d11=diff11(mx,C%diffusion)/(deltax(mx)/2.0)
    mat_dot_vec_pointer(ib)=mat_dot_vec_pointer(ib)-d11*vec(mx)*weight

  END FUNCTION mat_dot_vec_pointer

  FUNCTION mat_dot_vec_trans_pointer (mat,vec,siz)

    !-----------------------------------------------------------------------
    ! Goal: Calculate the product of transpose of matrix mat 
    !       and vector vec (of size siz) if vec is a pointer
    !-----------------------------------------------------------------------

    USE mes_types

    TYPE (diag),dimension(:),pointer :: mat
    REAL,DIMENSION(:),pointer :: vec
    INTEGER,INTENT(IN) :: siz

    INTEGER :: i,j,s
    REAL,DIMENSION(siz) :: mat_dot_vec_trans_pointer
    REAL :: d11

    s=size(mat)/2+1

    mat_dot_vec_trans_pointer=mat(1)%tab*vec

    do i=2,s
       j=s+i-1
       mat_dot_vec_trans_pointer(1+mat(i)%dist:siz)&
            &=mat_dot_vec_trans_pointer(1+mat(i)%dist:siz)&
            &+mat(i)%tab(1:siz-mat(i)%dist)*vec(1:siz-mat(i)%dist)

       mat_dot_vec_trans_pointer(1:siz-mat(j)%dist)&
            &=mat_dot_vec_trans_pointer(1:siz-mat(j)%dist)&
            &+mat(j)%tab(1+mat(j)%dist:siz)*vec(1+mat(j)%dist:siz)      
    end do

    i=ib
    d11=diff11(mx,C%diffusion)/(deltax(mx)/2.0)
    mat_dot_vec_trans_pointer(mx)=mat_dot_vec_trans_pointer(mx)&
         &-d11*vec(i)*weight

  END FUNCTION mat_dot_vec_trans_pointer

  FUNCTION scalar(vec1,vec2)

    !-----------------------------------------------------------------------
    ! Goal: Calculate the dot product of two 1D tables vec1 and vec2
    !-----------------------------------------------------------------------

    REAL,DIMENSION(1:) :: vec1,vec2
    REAL :: scalar

    scalar=SUM(vec1*vec2)

  END FUNCTION scalar


  FUNCTION scalar_pointer(vec1,vec2)

    !-----------------------------------------------------------------------
    ! Goal: Calculate the dot product of two 1D pointers vec1 and vec2
    !-----------------------------------------------------------------------

    REAL,DIMENSION(:),POINTER :: vec1,vec2
    REAL :: scalar_pointer

    scalar_pointer=SUM(vec1*vec2)

  END FUNCTION scalar_pointer


  function diff11 (i,tab)

    !-----------------------------------------------------------------------
    ! Goal: Calculate diff1 as the interpolate of the diffusion coefficient 
    !       at the interface between mesh i and mesh i+1
    !-----------------------------------------------------------------------

    integer,intent (in) :: i
    real :: ci,cio
    real :: diff11
    real,dimension(:),pointer :: tab

    ci=1.0-xinterp(i)
    cio=xinterp(i)

    diff11=ci*tab(i)+cio*tab(i+1)

  end function diff11

  function time_diff (time)

    !-----------------------------------------------------------------------
    ! Goal: Calculate the diffusion coefficient at time time in the time 
    !       dependent diffusion case
    !-----------------------------------------------------------------------

    real,intent(in) :: time
    real :: time_diff

    time_diff=.01+.09*abs(cos(2.0*pi*time))
    IF (time<0.0) time_diff=0.0


  end function time_diff

  function p_integral (x,f,xmin,xmax)

    !-----------------------------------------------------------------------
    ! Goal: Calculate the integral between xmin and xmax of the piecewise 
    !       constant function given by the abscissa in the 1D pointer x 
    !       and values in the 1D pointer f 
    !-----------------------------------------------------------------------

    real,intent(in) :: xmin,xmax
    real,dimension(:),pointer :: x,f
    real :: p_integral

    integer :: imin,imax,i,j1,j2

    p_integral=0.0


    IF (xmin+1e-15<xmax) THEN
       imin=1
       imax=ref%counter
       i=1
       loop1 : DO i=1,ref%counter
          if ((xmin>=x(i)).and.(xmin<x(i+1))) then
             imin=i
             exit loop1
          END if
       END DO loop1
       j1=i

       i=1
       loop2: DO i=1,ref%counter
          if ((xmax>x(i)).and.(xmax<=x(i+1))) then
             imax=i
             exit loop2
          END if
       END DO loop2
       j2=i

       DO i=imin+1,imax-1
          p_integral=p_integral+(x(i+1)-x(i))*f(i)
       END DO
       IF (imin/=imax) THEN
          p_integral=p_integral+(x(imin+1)-xmin)*f(imin)+(xmax-x(imax))*f(imax)
       ELSE
          p_integral=p_integral+(xmax-xmin)*f(imin)
       END IF
    END IF

    if ((imin>400).or.(imax>400)) then
       print*,'val=',imin,imax,x(imin),x(imax),xmin,xmax
       stop
    end if

  end function p_integral


END MODULE functions







