MODULE mes_types

  !--------------------------------------------------------------------------
  ! Goal : Set up derived types
  !--------------------------------------------------------------------------

  TYPE diag
     !-----------------------------------------------------------------------
     ! One diagonal of the diffusion matrix:
     !--------------------------------------
     ! dist: distance from the current diagonal to the main diagonal
     ! tab: coefficients on the current diagonal
     !-----------------------------------------------------------------------
     INTEGER :: dist
     REAL,DIMENSION(:),POINTER :: tab
  END TYPE diag


  TYPE refractory

     !-----------------------------------------------------------------------
     ! Parameters related to the refractory period:
     !---------------------------------------------
     ! time: time at which the solution is stored at the boundary for future 
     !  re-injection
     ! rflux: diffusive out-flux is stored for future re-injection
     ! counter: current index to write in tables time and rflux
     !-----------------------------------------------------------------------
     REAL, DIMENSION(:), POINTER :: time,rflux
     INTEGER :: counter

  END TYPE refractory


  TYPE cl

     !-----------------------------------------------------------------------
     ! Parameters related to the boundary conditions:
     !------------------------------------------------
     ! type: (1): Dirichlet; (2): Neumann; (3): No-flux
     ! rflux: diffusive out-flux is stored for future re-injection
     ! counter: current index to write in tables time and rflux
     !-----------------------------------------------------------------------
     INTEGER :: TYPE
     REAL :: val

  END TYPE cl



  TYPE Champ

     !-----------------------------------------------------------------------
     ! Numerical solution
     !------------------------------------------------
     ! tau: refractory period
     ! val: table of values of P
     ! diffusion: table of values of D
     ! left, right: left and right Boundary conditions
     !-----------------------------------------------------------------------
     REAL :: tau
     REAL, DIMENSION(:),POINTER :: val       !tableau de valeurs
     REAL, DIMENSION(:),POINTER :: diffusion
     TYPE(cl) :: left,right

  END TYPE Champ


END MODULE mes_types
