MODULE plot

  !----------------------------------------------------------------
  ! Goal : Manages the printing of the solution in the .vtk format
  !----------------------------------------------------------------

  USE files
  USE mes_types
  USE param
  USE functions

  IMPLICIT NONE

CONTAINS

  SUBROUTINE create_vtk

    !----------------------------------------------------------------
    ! Goal : Create the result file FPLOTPLT in the .vtk format
    !----------------------------------------------------------------

    INTEGER :: i,j,k,ind1,ind2,counter
    REAL :: absc

    counter=0
    DO i=1,mx
       absc=xpoints(i)
       IF ((absc>=xp_min).and.(absc<=xp_max)) counter=counter+1
    END DO

    OPEN (unit=uplotplt,file=FPLOTPLT,status='replace')
    WRITE(uplotplt,'(A)') '# vtk DataFile Version 2.0'
    WRITE(uplotplt,'(A)') 'Result'
    WRITE(uplotplt,'(A)') 'ASCII'
    WRITE(uplotplt,*) 'DATASET UNSTRUCTURED_GRID'
    WRITE(uplotplt,*) 'POINTS',counter+1,'float'

    !Added
!!$open (unit=55,file='x_coords',status='replace')
!!$DO i=mx,1,-1
!!$   write (55,400) -xpoints(i)+1.0
!!$END DO
!!$write (55,400) 1.0
!!$close(55)
!!$stop
    !------------------------------------

!!$      do i=mx,0,-1
!!$         absc=xmesh(i)
!!$         if (absc>2.0) absc=2.0+.5*absc/lx
!!$         write (uplotplt,500,advance='no') -(absc-1.0),0.0,0.0
!!$      end do

    ! Added
    DO i=1,mx
       absc=xpoints(i)
       IF ((absc>=xp_min).and.(absc<=xp_max))&
            &WRITE (uplotplt,500,advance='no') absc,0.0,0.0
    END DO
    WRITE (uplotplt,500,advance='no') xp_max,0.0,0.0

    CLOSE(uplotplt)

    OPEN (unit=uplotplt,file=FPLOTPLT,status='old',position='append')

    WRITE(uplotplt,*) 'CELLS',counter,counter*3

    DO i=1,counter
       WRITE (uplotplt,600) 2,i-1,i
    END DO

    WRITE(uplotplt,*) 'CELL_TYPES',counter
    DO i=1,counter
       WRITE(uplotplt,*) 3
    END DO

    WRITE(uplotplt,*) 'POINT_DATA',(counter+1)
!!$WRITE(uplotplt,*) 'CELL_DATA',mx

    CLOSE(uplotplt)

400 FORMAT (f10.5)
500 FORMAT (3(E14.7,3x))
600 FORMAT (9(I6,3X))

  END SUBROUTINE create_vtk


  SUBROUTINE plot_vtk (vec,chaine)

    !----------------------------------------------------------------
    ! Goal : Plot the result vec in the .vtk format
    !        into the file of unit uplotplt
    !----------------------------------------------------------------

    REAL,DIMENSION(:),POINTER :: vec
    CHARACTER(len=*),INTENT(in) :: chaine
    INTEGER :: i,j,k
    REAL :: absc

    OPEN (unit=uplotplt,file=FPLOTPLT,status='old',position='append')
    WRITE(uplotplt,*) 'SCALARS ',chaine,' float 1'
    WRITE(uplotplt,*) 'LOOKUP_TABLE default'

    ! Cell data
    !-----------
!!$  DO i=mx,1,-1
!!$     write (uplotplt,500) vec(i)
!!$  END DO

    ! Point data
    !------------

    ! Added
    DO i=1,mx
       absc=xpoints(i)
       IF ((absc>=xp_min).and.(absc<=xp_max)) &
            &WRITE (uplotplt,500) vec(i) 
    END DO
    WRITE (uplotplt,500) 0.0

!!$  i=nx-1
!!$  write (uplotplt,500) vec(i)
!!$  DO i=nx-1,ib+2,-1
!!$     write (uplotplt,500) (1.0-xinterp(i-1))*vec(i-1)+xinterp(i-1)*vec(i)
!!$  END DO
!!$  i=ib+1
!!$  !write (uplotplt,500) (deltax(ib)*deltax(ib+1)*vec(1)/deltax(1)&
!!$  !     &+deltax(ib+1)*vec(ib)+deltax(ib)*vec(ib+1))/dxp(ib)/2.0
!!$  write (uplotplt,500) vec(ib)+deltax(ib)/2.0*((vec(ib)-vec(ib-1))/deltax(ib-1))
!!$  DO i=ib,2,-1
!!$     write (uplotplt,500) (1.0-xinterp(i-1))*vec(i-1)+xinterp(i-1)*vec(i)
!!$  END DO
!!$  write (uplotplt,500) 0.0

    CLOSE (uplotplt)

500 FORMAT (f30.20)

  END SUBROUTINE plot_vtk

END MODULE plot
