SUBROUTINE transport (SB,dt)

  !---------------------------------------------------------------------
  ! Goal: Iterate the pure drift scheme
  !---------------------------------------------------------------------

  USE param
  USE inter

  IMPLICIT NONE

  REAL,INTENT(in) :: dt
  REAL,DIMENSION(:),POINTER :: SB
  REAL,DIMENSION(mx) :: sigmaxp,sigmaxm,kappaxp,kappaxm
  REAL,DIMENSION(mx) :: Lp,Lm

  INTEGER :: k,i,j
  REAL :: Vp,Vm,Wp,Wm,ps,Vx1,Vx2

  INTEGER :: opt

  opt=1

  SELECT CASE(schema_transport)

  CASE(1) 
     !---------------------------------------------------------------------
     ! Godunov scheme
     !---------------------------------------------------------------------

     !-------------------------------------------------------------------------
     ! Numerical fluxes
     !--------------------
     ! Interior interfaces
     !--------------------
     DO i=1,mx-1
        Vp=MAX(0.,Vx(i))
        Vm=MIN(0.,Vx(i))
        flux(i)=Vp*SB(i)+Vm*SB(i+1)
     END DO
     !---------------------------------
     ! Boundary interfaces (Left,right)
     !---------------------------------
!!$     i=mx
!!$     IF (C%right%type==1) THEN
!!$        flux(i)=Vx(i)*C%right%val
!!$     ELSE IF (C%right%type==2) THEN
!!$        flux(i)=0.0*Vx(i)*(SB(i)+C%right%val*deltax(i)/2.0)
!!$     ELSE IF (C%left%type==3) THEN
!!$        flux(i)=0.0
!!$     END IF
!!$
!!$     i=0
!!$     IF (C%left%type==1) THEN
!!$        flux(i)=Vx(i)*C%left%val
!!$     ELSE IF (C%left%type==2) THEN
!!$        flux(i)=Vx(i)*(SB(i)+C%left%val*deltax(i+1)/2.0)
!!$     ELSE IF (C%left%type==3) THEN
!!$        flux(i)=0.0
!!$     END IF
     flux(mx)=0.0
     flux(0)=0.0
     !------------------------------------------------------------------------

     ! Update the numerical solution at time tn+1
     !-------------------------------------------
     DO i=1,mx
        SB9(i)=SB(i)-dt/deltax(i)*(flux(i)-flux(i-1))
     END DO


  CASE(2) 
     !---------------------------------------------------------------------
     ! Godunov scheme with limiter
     !---------------------------------------------------------------------

     ! Coefficients sigma
     !--------------------
     DO i=1,mx-1
        Vp=MAX(0.,Vx(i))
        Vm=MIN(0.,Vx(i))
        sigmaxp(i)=Vp*(1.0-Vp*dt/deltax(i))
        sigmaxm(i)=Vm*(1.0+Vm*dt/deltax(i+1))
     END DO

     ! Limiters
     !---------
     DO i=2,Mx-1
        Wp = SB(i+1)-SB(i)
        Wm=SB(i)-SB(i-1)
        Vx1=MAX(0.,Vx(i-1))
        Vx2=MAX(0.,Vx(i))
        Lp(i) = limiter(Vx1*Wm,Vx2*Wp,dxp(i)/deltax(i),opt)
        Vx1=MIN(0.,Vx(i-1))
        Vx2=MIN(0.,Vx(i))
        Lm(i) = limiter(Vx2*Wp,Vx1*Wm,dxp(i-1)/deltax(i),opt)
     END DO
     Lp(1)=0.   ; Lp(Mx) = 0.
     Lm(1)=0.   ; Lm(Mx) = 0.

     !------------------------------------------------------------------------
     ! Numerical fluxes
     !------------------------------------------------------------------------
     ! Interior interfaces
     !--------------------
     DO i=1,mx-1
        Vp=MAX(0.,Vx(i))
        Vm=MIN(0.,Vx(i))
        Wp=(SB(i+1)-SB(i))/dxp(i)
        flux(i)=Vp*SB(i)+Vm*SB(i+1)
        flux(i)=flux(i)+.5&
             &*Wp*(Lp(i)*sigmaxp(i)*deltax(i)-Lm(i+1)*sigmaxm(i)*deltax(i+1))
     END DO

     !---------------------------------
     ! Boundary interfaces (left,right)
     !---------------------------------
!!$     i=mx
!!$     Vp=MAX(0.,Vx(i))
!!$     Vm=MIN(0.,Vx(i))
!!$     IF (C%right%type==1) THEN
!!$        flux(i)=Vm*C%right%val+Vp*SB(i)
!!$     ELSE IF (C%right%type==2) THEN
!!$        flux(i)=Vx(i)*(SB(i)+C%right%val*deltax(i)/2.0)
!!$     ELSE IF (C%right%type==3) THEN
!!$        flux(i)=0.0
!!$     END IF
!!$     i=0
!!$     Vp=MAX(0.,Vx(i))
!!$     Vm=MIN(0.,Vx(i))
!!$     IF (C%left%type==1) THEN
!!$        flux(i)=Vp*C%left%val+Vm*SB(i+1)
!!$     ELSE IF (C%left%type==2) THEN
!!$        flux(i)=Vx(i)*(SB(i+1)+C%left%val*deltax(i+1)/2.0)
!!$     ELSE IF (C%left%type==3) THEN
!!$        flux(i)=0.0
!!$     END IF

     flux(0)=0.0
     flux(mx)=0.0
     !----------------------------------------------------------------------

     ! Update the solution at time tn+1
     !----------------------------------
     DO i=1,mx
        SB9(i)=SB(i)-dt/deltax(i)*(flux(i)-flux(i-1))
     END DO

  END SELECT

  SB = SB9

CONTAINS

  FUNCTION limiter(w1,w2,bound,opt)
    REAL,INTENT(in)::w1,w2,bound
    INTEGER, INTENT(in) :: opt
    REAL           ::limiter

    SELECT CASE(opt)
    CASE(1)
       limiter= supb(W1,W2,bound)!+supb(W1,W2)-W1
    CASE(2)
       limiter = vanleer(W1,W2)!+vanleer(W1,W2)-W1
    END SELECT
  END FUNCTION limiter


  FUNCTION supb(x,y,bound)
    !----------------------------------------------------------------
    ! Superbee limiter
    !----------------------------------------------------------------
    REAL,INTENT(in):: x,y,bound
    REAL           :: supb, eps, sid,theta , Mroe

    eps = 1.D-10
    Mroe=2.0*bound

    IF (ABS(y)<eps) THEN
       theta=x/eps
    ELSE
       theta=x/y
    END IF

    supb=MAX(0.0,MIN(1.0,Mroe*theta),MIN(Mroe,theta))



  END FUNCTION supb


  FUNCTION vanleer(x,y)
    !-----------------------------------------------------------------
    ! Van Leer limiter
    !-----------------------------------------------------------------
    REAL,INTENT(in):: x,y
    REAL           :: vanleer, eps, sid, theta

    eps = 1.D-10

    IF (ABS(y)<eps) THEN
       theta=x/eps
    ELSE
       theta=x/y
    END IF

    vanleer = 0.
    IF (theta > eps ) vanleer = (2.0*theta/(1.0+theta))

  END FUNCTION vanleer


END SUBROUTINE transport
