MODULE systeme_lineaire

  USE mes_types
  USE param
  USE functions

  implicit none

CONTAINS

  SUBROUTINE syslin(mat,vec,sol,x0,siz,precis,opt,precond,num)

    INTEGER,INTENT(in) :: opt
    INTEGER,INTENT(in) :: siz,precond
    REAL,INTENT(IN) :: precis
    TYPE (diag),DIMENSION(:),POINTER :: mat
    REAL,DIMENSION(:),POINTER :: vec,x0
    REAL,DIMENSION(:),POINTER :: sol
    INTEGER,intent(in),DIMENSION(:) ::num
    INTEGER :: opt_local


    opt_local=opt

    IF ((precond==1).OR.(precond==3)) CALL precond_diag(mat,vec,x0,siz)
    IF ((opt==2).and.((precond==2).or.(precond==3))) opt_local=4
    IF ((opt==2).and.(precond==4)) opt_local=5

    SELECT CASE (opt_local)

    CASE (1) 
       CALL gradient_conjugue(mat,vec,sol,x0,siz,precis)
    CASE (2)
       CALL bi_gradient(mat,vec,sol,x0,siz,precis)
    CASE (3)
       CALL gmres(mat,vec,sol,x0,siz,precis)
    CASE (4)
       select case (active)
       case (1)
          CALL bigrad_lu_3(mat,vec,sol,x0,siz,precis,num)
       case (2)
          CALL bigrad_lu_5(mat,vec,sol,x0,siz,precis,num)
       END select
    CASE (5)
       CALL bigrad_ssor5(mat,vec,sol,x0,siz,precis,num)
    CASE default
       PRINT*,'Methode de resolution de systeme lineaire non programmee',opt_local
       STOP
    END SELECT

    !print*,'Sortie de syslin'
  END SUBROUTINE syslin

  SUBROUTINE gradient_conjugue(mat,vec,sol,x0,siz,precis)

    INTEGER,INTENT(in) :: siz
    REAL,INTENT(IN) :: precis
    TYPE (diag),DIMENSION(:),POINTER :: mat
    REAL,DIMENSION(:),POINTER :: vec,x0
    REAL,DIMENSION(:),POINTER :: sol

    INTEGER      :: i
    REAL   :: alfa,beta,residu,newresidu,tester
    REAL, DIMENSION(siz)    :: temp, res, desc  

    ! initialisation

    !print*,'Entree dans Gradient conjugue'

    i = 1 
    sol = x0 
    desc = vec - mat_dot_vec(mat,sol,siz)
    !print*,'temp=',mat_dot_vec(mat,sol,siz)
!stop
    res = desc  
    residu = scalar(res,res)
    tester=SQRT(residu)
    !print*,'tester=',tester
    IF (tester < precis) RETURN

    ! debut des iterations

    DO

       !print*,'i=',i

       temp = mat_dot_vec(mat,desc,siz)
       alfa = residu/scalar(temp,desc)
       sol = sol + (alfa * desc)
       res = res - (alfa * temp)
       newresidu =scalar(res,res)
       beta = newresidu/residu
       desc = res + (beta * desc)
       residu = newresidu
       tester=SQRT(residu/scalar(sol,sol))
       i = i + 1
       !print*,'i=',i,tester

       ! sortie

       IF( tester < precis ) EXIT
       IF( i > siz+10) THEN
          PRINT *, 'GC ne converge pas -> n_iter =', i, 'residu =', residu
          STOP
       END IF
    END DO

    !print*,'Sortie de Gradient conjugue'

  END SUBROUTINE gradient_conjugue

  SUBROUTINE bi_gradient(mat,vec,sol,x0,siz,precis)

    INTEGER,INTENT(in) :: siz
    REAL,INTENT(IN) :: precis
    TYPE(diag),DIMENSION(:),POINTER :: mat
    REAL,DIMENSION(:),POINTER :: vec,x0
    REAL,DIMENSION(:),POINTER :: sol

    INTEGER      :: i,j,k,l
    REAL   :: alfa,beta,residu,residu_bis,norm,tester
    REAL, DIMENSION(siz)    :: temp, desc1,res2,desc2,res1

    !print*,'Entree dans bi_gradient'

    norm=SQRT(scalar_pointer(vec,vec))

    tester=norm
    i = 1 
    sol = x0 
    desc1 = vec - mat_dot_vec_pointer(mat,sol,siz)
    desc2 = vec - mat_dot_vec_trans_pointer(mat,sol,siz) 
    res1 = desc1
    res2 = desc2
    residu_bis = scalar(res1,res2)
    residu = scalar(res1,res1)
    tester=SQRT(residu/norm)
    IF (tester < precis) RETURN
    
    ! debut des iterations

    DO
       temp = mat_dot_vec(mat,desc1,siz)
       alfa = residu_bis/scalar(temp,desc2)
       sol = sol + (alfa * desc1)
       res1 = res1 - (alfa * temp)
       res2 = res2 - (alfa * mat_dot_vec_trans(mat,desc2,siz))
       beta = scalar(res1,res2)/residu_bis
       desc1 = res1+(beta * desc1)
       desc2 = res2 + (beta * desc2)
       residu_bis=scalar(res1,res2)
       residu =scalar(res1,res1)
       !print*,'residu=',residu
       tester=SQRT(residu/scalar_pointer(sol,sol))
       ! sortie
       IF( tester < precis ) EXIT
       IF( i > siz+10) THEN
          PRINT *, 'bi_gradient ne converge pas -> n_iter ='&
               &, i, 'residu =', tester
          STOP
       END IF

       i = i + 1

    END DO

    !print*,'Sortie de bi_gradient'

  END SUBROUTINE bi_gradient


  SUBROUTINE gmres(mat,vec,sol,x0,siz,precis)

    INTEGER,INTENT(in) :: siz
    REAL,INTENT(IN) :: precis
    TYPE (diag),DIMENSION(:),POINTER :: mat
    REAL,DIMENSION(:),POINTER :: vec,x0
    REAL,DIMENSION(:),POINTER :: sol

    INTEGER      :: i
    REAL   :: alfa,beta,sav,tester
    REAL, DIMENSION(siz)    :: temp, res, desc1,desc2

    i=0
    sol=x0
    res=vec-mat_dot_vec(mat,sol,siz)
    desc1=res
    desc2=mat_dot_vec(mat,desc1,siz)

    DO
       tester=SQRT(scalar(res,res)/scalar(sol,sol))
       IF (tester<precis) EXIT
       IF (i>siz+10) THEN
          PRINT*,'Divergence GMRES i=',i,'residu=',SQRT(scalar(res,res))
          call desallocation
          stop
       END IF
       i=i+1

       sav=scalar(desc2,desc2)
       alfa=scalar(res,desc2)/sav
       sol=sol+alfa*desc1
       res=res-alfa*desc2
       temp=mat_dot_vec(mat,res,siz)
       beta=scalar(temp,desc2)/sav
       desc1=res+beta*desc1
       desc2=temp+beta*desc2
    END DO

  END SUBROUTINE gmres

  SUBROUTINE precond_diag(mat,vec,x0,siz)

    INTEGER,INTENT(in) :: siz
    TYPE (diag),DIMENSION(:),POINTER :: mat
    REAL,DIMENSION(:),POINTER :: vec,x0

    INTEGER :: i,j
    REAL :: val

!!$    do j=1,SIZE(mat)
!!$do i=1,siz
!!$print*,j,mat(j)%tab(i)
!!$
!!$end do
!!$end do

    DO i=1,siz
       val=1.0/mat(1)%tab(i)
       DO j=1,SIZE(mat)
          mat(j)%tab(i)=mat(j)%tab(i)*val
       END DO
       vec(i)=vec(i)*val
       x0(i)=x0(i)*val
    END DO

!!$do j=1,SIZE(mat)
!!$do i=1,siz
!!$print*,j,mat(j)%tab(i)
!!$
!!$end do
!!$end do

!stop
  END SUBROUTINE precond_diag




  SUBROUTINE bigrad_lu_5 (mat,vec,sol,x0,siz,precis,num)

    INTEGER,INTENT(in) :: siz
    REAL,INTENT(IN) :: precis
    TYPE (diag),DIMENSION(:),POINTER :: mat
    REAL,DIMENSION(:),POINTER :: vec,x0
    REAL,DIMENSION(:),POINTER :: sol
    INTEGER,intent(in),dimension(5) :: num

    REAL,DIMENSION(3,siz) :: TI,SU

    INTEGER      :: i
    REAL   :: alfa,beta,sav1,sav2,residu
    REAL, DIMENSION(siz)    :: temp1,temp2,res1,res2, desc1,desc2,H1,H2

    !print*,'Entree dans bigrad_lu'

    !temp1=0.0
    !temp2=0.0
    !res1=0.0
    !res2=0.0
    !desc1=0.0
    !desc2=0.0
    H1=0.0
    H2=0.0
    TI=0.0
    SU=0.0
    residu=0.0
    sav1=0.0
    sav2=0.0
    i=0

    CALL inc_gauss_5(mat,TI,SU,siz,num)

    sol=x0

    res1 = vec-mat_dot_vec_pointer(mat,sol,siz)
    res2 = vec-mat_dot_vec_trans_pointer(mat,sol,siz)

    CALL trisup_5 (TI,SU,H1,res1,mat(num(3))%dist,siz)

    CALL trisup_trans_5 (TI,SU,H2,res2,mat(num(3))%dist,siz)

    desc1=H1
    desc2=H2
    sav1=scalar(H1,res2)

!!$    residu=SQRT(scalar(res1,res1))
!!$    IF (residu<precis) RETURN

    DO
       residu=SQRT(scalar(res1,res1))
       !SQRT(scalar(res1,res1)/scalar_pointer(sol,sol))
       
       IF (residu<precis) EXIT
       !IF (ABS(sav1)<precis) EXIT

       i=i+1
       !print*,'i=',i,SQRT(scalar(res1,res1)/scalar_pointer(sol,sol))
       temp1 = mat_dot_vec(mat,desc1,siz)
       alfa = sav1/scalar(desc2,temp1)
!!$       solp=alfa*desc1
!!$       sol = sol+solp
!!$       residu=maxval(abs(solp))
!!$       IF (residu<precis) EXIT
       sol = sol+alfa*desc1
       res1 = res1-alfa*temp1
       temp2 = mat_dot_vec_trans(mat,desc2,siz)
       res2 = res2-alfa*temp2

       CALL trisup_5 (TI,SU,H1,res1,mat(num(3))%dist,siz)
       CALL trisup_trans_5 (TI,SU,H2,res2,mat(num(3))%dist,siz)

       sav2 = scalar(H1,res2)
       beta = sav2/sav1
       sav1 = sav2
       desc1 = H1+beta*desc1
       desc2 = H2+beta*desc2

    END DO

    !print*,'Sortie de bigrad_lu'
  end SUBROUTINE bigrad_lu_5

  SUBROUTINE bigrad_ssor5 (mat,vec,sol,x0,siz,precis,num)

    INTEGER,INTENT(in) :: siz
    REAL,INTENT(IN) :: precis
    TYPE (diag),DIMENSION(:),POINTER :: mat
    REAL,DIMENSION(:),POINTER :: vec,x0
    REAL,DIMENSION(:),POINTER :: sol
    INTEGER,intent(in),dimension(5) :: num

    REAL,DIMENSION(3,siz) :: TI,SU

    INTEGER      :: i
    REAL   :: alfa,beta,sav1,sav2,residu
    REAL, DIMENSION(siz)    :: temp1,temp2,res1,res2, desc1,desc2,H1,H2

    !print*,'Entree dans bigrad_lu'

    !temp1=0.0
    !temp2=0.0
    !res1=0.0
    !res2=0.0
    !desc1=0.0
    !desc2=0.0
    H1=0.0
    H2=0.0
    TI=0.0
    SU=0.0
    residu=0.0
    sav1=0.0
    sav2=0.0
    i=0

    CALL inc_ssor_5(mat,TI,SU,siz,num)

    sol=x0

    res1 = vec-mat_dot_vec_pointer(mat,sol,siz)
    res2 = vec-mat_dot_vec_trans_pointer(mat,sol,siz)

    CALL trisup_5 (TI,SU,H1,res1,mat(num(3))%dist,siz)

    CALL trisup_trans_5 (TI,SU,H2,res2,mat(num(3))%dist,siz)

    desc1=H1
    desc2=H2
    sav1=scalar(H1,res2)

!!$    residu=SQRT(scalar(res1,res1))
!!$    IF (residu<precis) RETURN

    DO
       residu=SQRT(scalar(res1,res1))
       !SQRT(scalar(res1,res1)/scalar_pointer(sol,sol))
       
       IF (residu<precis) EXIT
       !IF (ABS(sav1)<precis) EXIT

       i=i+1
       !print*,'i=',i,SQRT(scalar(res1,res1)/scalar_pointer(sol,sol))
       temp1 = mat_dot_vec(mat,desc1,siz)
       alfa = sav1/scalar(desc2,temp1)
!!$       solp=alfa*desc1
!!$       sol = sol+solp
!!$       residu=maxval(abs(solp))
!!$       IF (residu<precis) EXIT
       sol = sol+alfa*desc1
       res1 = res1-alfa*temp1
       temp2 = mat_dot_vec_trans(mat,desc2,siz)
       res2 = res2-alfa*temp2

       CALL trisup_5 (TI,SU,H1,res1,mat(num(3))%dist,siz)
       CALL trisup_trans_5 (TI,SU,H2,res2,mat(num(3))%dist,siz)

       sav2 = scalar(H1,res2)
       beta = sav2/sav1
       sav1 = sav2
       desc1 = H1+beta*desc1
       desc2 = H2+beta*desc2

    END DO

    !print*,'Sortie de bigrad_lu'
  end SUBROUTINE bigrad_ssor5


  SUBROUTINE inc_gauss_5 (mat,TI,SU,siz,num)

    INTEGER,INTENT(in) :: siz
    TYPE(diag),DIMENSION(:),POINTER :: mat
    REAL,DIMENSION(:,:),INTENT(out) :: TI,SU
    INTEGER,intent(in),DIMENSION(5) :: num


    INTEGER :: i

    !print*,'Entree dans inc_gauss'

    TI=0.0
    SU=0.0

    TI(1,1:siz) = 1.0
    SU(2,1:siz-1) = mat(num(2))%tab(1:siz-1)
    SU(3,1:siz-mat(num(3))%dist) = mat(num(3))%tab(1:siz-mat(num(3))%dist)
    SU(1,1) = mat(num(1))%tab(1)

    DO i=2,mat(num(3))%dist
       TI(2,i) = mat(num(4))%tab(i)/SU(1,i-1)
       SU(1,i) = mat(num(1))%tab(i)-TI(2,i)*SU(2,i-1)
    END DO

    DO i=1+mat(num(3))%dist,siz
       TI(2,i) = mat(num(4))%tab(i)/SU(1,i-1)
       TI(3,i) = mat(num(5))%tab(i)/SU(1,i-mat(num(3))%dist)
       SU(1,i) = mat(num(1))%tab(i)-TI(3,i)*SU(3,i)-TI(2,i)*SU(2,i-1)
    END DO

    !print*,'Sortie de inc_gauss'

  END SUBROUTINE inc_gauss_5

  SUBROUTINE inc_ssor_5 (mat,TI,SU,siz,num)

    INTEGER,INTENT(in) :: siz
    TYPE(diag),DIMENSION(:),POINTER :: mat
    REAL,DIMENSION(:,:),INTENT(inout) :: TI,SU
    INTEGER,intent(in),DIMENSION(5) :: num

    REAL :: omega

    omega=1.25

    SU(1,:)=1.0/omega
    SU(2,:)=mat(num(2))%tab/mat(1)%tab
    SU(3,:)=mat(num(3))%tab/mat(1)%tab
    !SU(4,:)=mat(num(4))%tab/mat(1)%tab

    SU=omega/(2.0-omega)*SU

    TI(1,:)=mat(1)%tab/omega
    TI(2,:)=mat(num(4))%tab
    TI(3,:)=mat(num(5))%tab
    !TI(4,:)=mat(num(7))%tab

  END SUBROUTINE inc_ssor_5



  SUBROUTINE trisup_5 (TI,SU,H1,R1,dist,siz)

    INTEGER,INTENT(in) :: dist,siz
    REAL,DIMENSION(:,:),INTENT(in) :: TI,SU
    REAL,DIMENSION(:),INTENT(in) :: R1
    REAL,DIMENSION(:),INTENT(inout) :: H1

    INTEGER :: i
    REAL,DIMENSION(siz) :: X1

    ! Calcul de X1 tel que TI*X1 = R1
    !--------------------------------

    X1 = 0.0

    X1(1) = R1(1)/TI(1,1)

    DO i=2,dist
       X1(i) = (R1(i)-TI(2,i)*X1(i-1))/TI(1,i)
    END DO

    DO i=dist+1,siz
       X1(i) = (R1(i)-TI(3,i)*X1(i-dist)-TI(2,i)*X1(i-1))/TI(1,i)
    END DO

    ! Calcul de H1 tel que SU*H1 = X1 et donc mat*H1 = TI*SU*H1 = R1
    !---------------------------------------------------------------

    H1=0.0

    H1(siz) = X1(siz)/SU(1,siz)

    DO i=siz-1,siz-dist+1,-1
       H1(i) = (X1(i)-SU(2,i)*H1(i+1))/SU(1,i)
    END DO

    DO i=siz-dist,1,-1
       H1(i) = (X1(i)-SU(3,i)*H1(i+dist)-SU(2,i)*H1(i+1))/SU(1,i)
    END DO

  END SUBROUTINE trisup_5


  SUBROUTINE trisup_trans_5 (TI,SU,H1,R1,dist,siz)

    INTEGER,INTENT(in) :: dist,siz
    REAL,DIMENSION(:,:),INTENT(in) :: TI,SU
    REAL,DIMENSION(:),INTENT(in) :: R1
    REAL,DIMENSION(:),INTENT(inout) :: H1

    INTEGER :: i
    REAL,DIMENSION(siz) :: X1

    ! Calcul de X1 tel que tSU*X1 = R1
    !---------------------------------

    X1=0.0

    X1(1) = R1(1)/SU(1,1)

    DO i=2,dist
       X1(i) = (R1(i)-SU(2,i-1)*X1(i-1))/SU(1,i)
    END DO

    DO i=1+dist,siz
       X1(i) = (R1(i)-SU(3,i-dist)*X1(i-dist)-SU(2,i-1)*X1(i-1))/SU(1,i)
    END DO

    ! Calcul de H1 tel que tTI*H1 = X1 et donc tmat*H1 = tSU*tTI*H1 = R1
    !-------------------------------------------------------------------

    H1=0.0

    H1(siz) = X1(siz)/TI(1,siz)

    DO i=siz-1,siz-dist+1,-1
       H1(i) = (X1(i)-TI(2,i+1)*H1(i+1))/TI(1,i)
    END DO

    DO i=siz-dist,1,-1
       H1(i) = (X1(i)-TI(3,i+dist)*H1(i+dist)-TI(2,i+1)*H1(i+1))/TI(1,i)
    END DO

  END SUBROUTINE trisup_trans_5


  SUBROUTINE bigrad_lu_3 (mat,vec,sol,x0,siz,precis,num)

    !----------------------------------------------------------------------
    ! CONJUGATE BI-GRADIENT WITH LU PRE-CONDITIONING
    !----------------------------------------------------------------------

    INTEGER,INTENT(in) :: siz
    REAL,INTENT(IN) :: precis
    TYPE (diag),DIMENSION(:),POINTER :: mat
    REAL,DIMENSION(:),POINTER :: vec,x0
    REAL,DIMENSION(:),POINTER :: sol
    INTEGER,INTENT(in),DIMENSION(7) :: num

    REAL,DIMENSION(2,siz) :: TI,SU

    INTEGER      :: i,j
    REAL   :: alfa,beta,sav1,sav2,residu,omega
    REAL, DIMENSION(siz)    :: temp1,temp2,res1,res2, desc1,desc2,H1,H2

    H1=0.0
    H2=0.0
    TI=0.0
    SU=0.0
    residu=0.0
    sav1=0.0
    sav2=0.0
    i=0

    CALL inc_gauss_3(mat,TI,SU,siz,num)

    sol=x0

    res1 = vec-mat_dot_vec_pointer(mat,sol,siz)
    res2 = vec-mat_dot_vec_trans_pointer(mat,sol,siz)

    CALL trisup_3 (TI,SU,H1,res1,mat(num(3))%dist,mat(num(4))%dist,siz)

    CALL trisup_trans_3 (TI,SU,H2,res2,mat(num(3))%dist,mat(num(4))%dist,siz)

    desc1=H1
    desc2=H2
    sav1=scalar(H1,res2)

    DO
       residu=MAXVAL(ABS(res1))
       IF (residu<precis) EXIT
       i=i+1
       temp1 = mat_dot_vec(mat,desc1,siz)
       alfa = sav1/scalar(desc2,temp1)
       sol = sol+alfa*desc1
       res1 = res1-alfa*temp1
       temp2 = mat_dot_vec_trans(mat,desc2,siz)
       res2 = res2-alfa*temp2

       CALL trisup_3 (TI,SU,H1,res1,mat(num(3))%dist,mat(num(4))%dist,siz)
       CALL trisup_trans_3 (TI,SU,H2,res2,mat(num(3))%dist,mat(num(4))%dist,siz)

       sav2 = scalar(H1,res2)
       beta = sav2/sav1
       sav1 = sav2
       desc1 = H1+beta*desc1
       desc2 = H2+beta*desc2

    END DO

  END SUBROUTINE bigrad_lu_3


  SUBROUTINE inc_gauss_3 (mat,TI,SU,siz,num)

    !-------------------------------------------------------------------
    ! Create matrices L and U
    !-------------------------------------------------------------------

    INTEGER,INTENT(in) :: siz
    TYPE(diag),DIMENSION(:),POINTER :: mat
    REAL,DIMENSION(:,:),INTENT(inout) :: TI,SU
    INTEGER,INTENT(in),DIMENSION(7) :: num

    INTEGER :: i,mp,ms

    mp=mat(num(2))%dist
    
    TI=0.0
    SU=0.0

    TI(1,1:siz) = 1.0
    SU(2,1:siz-1) = mat(num(2))%tab(1:siz-1)
    SU(1,1) = mat(num(1))%tab(1)

    DO i=2,siz
       TI(2,i) = mat(num(3))%tab(i)/SU(1,i-1)
       SU(1,i) = mat(num(1))%tab(i)-TI(2,i)*SU(2,i-1)
    END DO


  END SUBROUTINE inc_gauss_3


  SUBROUTINE trisup_3 (TI,SU,H1,R1,mp,ms,siz)

    !--------------------------------------------------------------------
    ! Goal: solve upper and lower triangular linear systems corresponding
    !       with LU decomposition
    !-------------------------------------------------------------------

    INTEGER,INTENT(in) :: mp,ms,siz
    REAL,DIMENSION(:,:),INTENT(inout) :: TI,SU
    REAL,DIMENSION(:),INTENT(inout) :: R1
    REAL,DIMENSION(:),INTENT(inout) :: H1

    INTEGER :: i
    REAL,DIMENSION(siz) :: X1


    ! Compute de X1 such that TI*X1 = R1
    !--------------------------------
    X1(1) = R1(1)/TI(1,1)
    DO i=2,siz
       X1(i) = (R1(i)-TI(2,i)*X1(i-1))/TI(1,i)
    END DO

    ! Compute H1 such that SU*H1 = X1 and then mat*H1 = TI*SU*H1 = R1
    !----------------------------------------------------------------
    H1(siz) = X1(siz)/SU(1,siz)
    DO i=siz-1,1,-1
       H1(i) = (X1(i)-SU(2,i)*H1(i+1))/SU(1,i)
    END DO

  END SUBROUTINE trisup_3


  SUBROUTINE trisup_trans_3 (TI,SU,H1,R1,mp,ms,siz)

    !-----------------------------------------------------------------------
    ! Goal: solve the transpose of lower and upper triangular linear systems
    !       corresponding with LU decomposition
    !-----------------------------------------------------------------------

    INTEGER,INTENT(in) :: mp,ms,siz
    REAL,DIMENSION(:,:),INTENT(in) :: TI,SU
    REAL,DIMENSION(:),INTENT(in) :: R1
    REAL,DIMENSION(:),INTENT(inout) :: H1

    INTEGER :: i
    REAL,DIMENSION(siz) :: X1

    ! Compute de X1 such that tSU*X1 = R1
    !------------------------------------
    X1(1) = R1(1)/SU(1,1)
    DO i=2,siz
       X1(i) = (R1(i)-SU(2,i-1)*X1(i-1))/SU(1,i)
    END DO

    ! Compute H1 such that tTI*H1 = X1 and then tmat*H1 = tSU*tTI*H1 = R1
    !--------------------------------------------------------------------
    H1(siz) = X1(siz)/TI(1,siz)
    DO i=siz-1,1,-1
       H1(i) = (X1(i)-TI(2,i+1)*H1(i+1))/TI(1,i)
    END DO
    
  END SUBROUTINE trisup_trans_3


END MODULE systeme_lineaire
