SUBROUTINE transport (SB,dt)

  USE param
  USE inter
  USE functions

  implicit none

  !INTEGER,INTENT(IN) :: par
  REAL,INTENT(in) :: dt
  REAL,dimension(:,:),pointer :: SB
  REAL,dimension(mx,my) :: sigmaxp,sigmaxm,sigmayp,sigmaym
  REAL,dimension(mx,my) :: Lpx,Lmx
  REAL,dimension(mx,my) :: Lpy,Lmy
  !REAL,dimension(0:mx,1:my) :: xflux
  !REAL,dimension(1:mx,0:my) :: yflux

  INTEGER :: k,i,j
  real :: Vp,Vm,Wp,Wm,ps,Vx1,Vx2

  integer :: opt

  opt=1

  
  ! les vitesses aux interfaces pour une maille (i,j,k) sont notees:
  ! Vx(i,j,k) : cote droit
  ! Vy(i,j,k) : cote arriere
  ! Vz(i,j,k) : cote haut
  !print*,'Entree dans transport'


  select case(schema_transport)
  case(1) ! schema de Murmann 

     ! Direction x
     !-------------
     DO j=1,my
        DO i=1,mx-1
           Vp=max(0.,Vx(i,j))
           Vm=min(0.,Vx(i,j))
           xflux(i,j)=Vp*SB(i,j)+Vm*SB(i+1,j)
        END DO
     END DO

     !CL Droit, gauche
     i=mx
     DO j=1,my
        Vp=max(0.,Vx(i,j))
        Vm=min(0.,Vx(i,j))
        if (C%right(j)%type==1) then
           xflux(i,j)=Vm*C%right(j)%val+Vp*SB(i,j)
        else if (C%right(j)%type==2) then
           xflux(i,j)=Vx(i,j)*(SB(i,j)+C%right(j)%val*deltax(i)/2.0)
        else if (C%left(j)%type==3) then
           xflux(i,j)=0.0
        end if
     END DO
     i=0
     DO j=1,my
        Vp=max(0.,Vx(i,j))
        Vm=min(0.,Vx(i,j))
        if (C%left(j)%type==1) then
           xflux(i,j)=Vp*C%left(j)%val+Vm*SB(i+1,j)
        else if (C%left(j)%type==2) then
           xflux(i,j)=max(0.0,Vx(i,j))*(SB(i+1,j)+C%left(j)%val&
                &*deltax(i+1)/2.0)
        else if (C%left(j)%type==3) then
           xflux(i,j)=0.0
        end if
     END DO


     ! Direction y
     !-------------
     DO j=1,my-1
        DO i=1,mx
           Vp=max(0.,Vy(i,j))
           Vm=min(0.,Vy(i,j))
           yflux(i,j)=Vp*SB(i,j)+Vm*SB(i,j+1)
        END DO
     END DO

     !CL Top, Bottom
     j=my
     DO i=1,mx
        Vp=max(0.,Vy(i,j))
        Vm=min(0.,Vy(i,j))
        if (C%top(i)%type==1) then
           yflux(i,j)=Vm*C%top(i)%val+Vp*SB(i,j)
        else if (C%top(i)%type==2) then
           yflux(i,j)=Vy(i,j)*(SB(i,j)+C%top(i)%val*deltay(j)/2.0)
        else if (C%top(i)%type==3) then
           yflux(i,j)=0.0
        end if
     END DO

     j=0
     DO i=1,mx
        Vp=max(0.,Vy(i,j))
        Vm=min(0.,Vy(i,j))
        if (C%bottom(i)%type==1) then
         yflux(i,j)=Vp*C%bottom(i)%val+Vm*SB(i,j+1)
      else if (C%bottom(i)%type==2) then
           yflux(i,j)=Vy(i,j)*(SB(i,j+1)+C%bottom(i)%val*deltay(j+1)/2.0)
        else if (C%top(i)%type==3) then
           yflux(i,j)=0.0
        end if
     END DO

     ! No-convective-flux enforced
     !----------------------------
     xflux(0,:)=0.0
     yflux(:,0)=0.0
     xflux(mx,:)=0.0
     yflux(:,my)=0.0


     ! Obtain the updated solution
     !----------------------------
     do j=1,my
        do i=1,mx
           SB9(i,j)=SB(i,j)-dt/deltax(i)*(xflux(i,j)-xflux(i-1,j))&
                &-dt/deltay(j)*(yflux(i,j)-yflux(i,j-1))
        end do
     end do

     SB = SB9


  case(2) !! Godunov
     
     ! Coefficients sigma
     !--------------------
     !if (iter==1) then 
     DO j=1,my       
        DO i=1,mx-1
           Vp=max(0.,Vx(i,j))
           Vm=min(0.,Vx(i,j))
           sigmaxp(i,j)=Vp*(1.0)!-1.0*Vp*dt/deltax(i))!Vp*(1.0-Vp*dt/deltax(i))
           sigmaxm(i,j)=Vm*(1.0)!+1.0*Vm*dt/deltax(i+1))!Vm*(1.0+Vm*dt/deltax(i+1))
        END DO
     END DO

     DO j=1,my-1       
        DO i=1,mx
           Vp=max(0.,Vy(i,j))
           Vm=min(0.,Vy(i,j))
           sigmayp(i,j)=Vp*(1.0)!-1.0*Vp*dt/deltay(j))!Vp*(1.0-Vp*dt/deltay(j))
           sigmaym(i,j)=Vm*(1.0)!+1.0*Vm*dt/deltay(j+1))!Vm*(1.0+Vm*dt/deltay(j+1))
        END DO
     END DO


     ! Limiters direction x
     !----------------------
     DO j=1,my
        do i=2,Mx-1
           Wp = SB(i+1,j)-SB(i,j)
           Wm=SB(i,j)-SB(i-1,j)
           Vx1=max(0.,Vx(i-1,j))
           Vx2=max(0.,Vx(i,j))
           Lpx(i,j) = limiter(Vx1*Wm,Vx2*Wp,dxp(i)/deltax(i),opt)
           Vx1=min(0.,Vx(i-1,j))
           Vx2=min(0.,Vx(i,j))
           Lmx(i,j) = limiter(Vx2*Wp,Vx1*Wm,dxp(i-1)/deltax(i),opt)
        end do
     END DO

     Lpx(1,:)=0.   ; Lpx(Mx,:) = 0.
     Lmx(1,:)=0.   ; Lmx(Mx,:) = 0.

     

     ! Limiters direction y
     !----------------------
     DO j=2,my-1
        do i=1,Mx
           Wp = SB(i,j+1)-SB(i,j)
           Wm=SB(i,j)-SB(i,j-1)
           Vx1=max(0.,Vy(i,j-1))
           Vx2=max(0.,Vy(i,j))
           Lpy(i,j) = limiter(Vx1*Wm,Vx2*Wp,dyp(j)/deltay(j),opt)
           Vx1=min(0.,Vy(i,j-1))
           Vx2=min(0.,Vy(i,j))
           Lmy(i,j) = limiter(Vx2*Wp,Vx1*Wm,dyp(j-1)/deltay(j),opt)
        end do
     END DO

     Lpy(:,1)=0.   ; Lpy(:,My) = 0.
     Lmy(:,1)=0.   ; Lmy(:,My) = 0.

     

     ! Numerical fluxes Godunov
     !--------------------------
     DO j=1,my
        DO i=1,mx-1
           Vp=max(0.,Vx(i,j))
           Vm=min(0.,Vx(i,j))
           Wp=(SB(i+1,j)-SB(i,j))/dxp(i)
           xflux(i,j)=Vp*SB(i,j)+Vm*SB(i+1,j)
           xflux(i,j)=xflux(i,j)+.5&
                &*Wp*(Lpx(i,j)*sigmaxp(i,j)*deltax(i)&
                &-Lmx(i+1,j)*sigmaxm(i,j)*deltax(i+1))
        END DO
     END DO

     DO j=1,my-1
        DO i=1,mx
           Vp=max(0.,Vy(i,j))
           Vm=min(0.,Vy(i,j))
           Wp=(SB(i,j+1)-SB(i,j))/dyp(j)
           yflux(i,j)=Vp*SB(i,j)+Vm*SB(i,j+1)
           yflux(i,j)=yflux(i,j)+.5&
                &*Wp*(Lpy(i,j)*sigmayp(i,j)*deltay(j)&
                &-Lmy(i,j+1)*sigmaym(i,j)*deltay(j+1))
        END DO
     END DO



     !CL Droit, gauche
     i=mx
     DO j=1,my
        Vp=max(0.,Vx(i,j))
        Vm=min(0.,Vx(i,j))
        if (C%right(j)%type==1) then
           xflux(i,j)=Vm*C%right(j)%val+Vp*SB(i,j)
        else if (C%right(j)%type==2) then
           xflux(i,j)=Vx(i,j)*(SB(i,j)+C%right(j)%val*deltax(i)/2.0)
        else if (C%left(j)%type==3) then
           xflux(i,j)=0.0
        end if
        xflux(i,j)=0.0
     END DO
     i=0
     DO j=1,my
        Vp=max(0.,Vx(i,j))
        Vm=min(0.,Vx(i,j))
        if (C%left(j)%type==1) then
           xflux(i,j)=Vp*C%left(j)%val+Vm*SB(i+1,j)
        else if (C%left(j)%type==2) then
           xflux(i,j)=max(0.0,Vx(i,j))*(SB(i+1,j)+C%left(j)%val&
                &*deltax(i+1)/2.0)
        else if (C%left(j)%type==3) then
           xflux(i,j)=0.0
        end if
        xflux(i,j)=0.0
     END DO

     !CL Top, Bottom
     j=my
     DO i=1,mx
        Vp=max(0.,Vy(i,j))
        Vm=min(0.,Vy(i,j))
        if (C%top(i)%type==1) then
           yflux(i,j)=Vm*C%top(i)%val+Vp*SB(i,j)
        else if (C%top(i)%type==2) then
           yflux(i,j)=Vy(i,j)*(SB(i,j)+C%top(i)%val*deltay(j)/2.0)
        else if (C%top(i)%type==3) then
           yflux(i,j)=0.0
        end if
        yflux(i,j)=0.0
     END DO

     j=0
     DO i=1,mx
        Vp=max(0.,Vy(i,j))
        Vm=min(0.,Vy(i,j))
        if (C%bottom(i)%type==1) then
         yflux(i,j)=Vp*C%bottom(i)%val+Vm*SB(i,j+1)
      else if (C%bottom(i)%type==2) then
           yflux(i,j)=Vy(i,j)*(SB(i,j+1)+C%bottom(i)%val*deltay(j+1)/2.0)
        else if (C%top(i)%type==3) then
           yflux(i,j)=0.0
        end if
        yflux(i,j)=0.0
     END DO

     
     
     ! No-convective-flux enforced
     !----------------------------
     xflux(0,:)=0.0
     yflux(:,0)=0.0
     xflux(mx,:)=0.0
     yflux(:,my)=0.0

     ! Obtain the updated solution
     !----------------------------
     do j=1,my
        do i=1,mx
           SB9(i,j)=SB(i,j)-dt/deltax(i)*(xflux(i,j)-xflux(i-1,j))&
                &-dt/deltay(j)*(yflux(i,j)-yflux(i,j-1))
        end do
     end do

     SB = SB9

  end select

  !print*,'Sortie de transport'

END SUBROUTINE transport
